@@ -46,11 +46,12 @@ public interface BookKeeperServerStats {
     // Stats
     String ADD_ENTRY_REQUEST = "ADD_ENTRY_REQUEST";
     String ADD_ENTRY = "ADD_ENTRY";
-    String WRITE_THREAD_QUEUED_LATENCY = "WRITE_THREAD_QUEUED_LATENCY";
+    String ADD_ENTRY_REJECTED = "ADD_ENTRY_REJECTED";
     String FORCE_LEDGER_REQUEST = "FORCE_LEDGER_REQUEST";
     String FORCE_LEDGER = "FORCE_LEDGER";
     String READ_ENTRY_REQUEST = "READ_ENTRY_REQUEST";
     String READ_ENTRY = "READ_ENTRY";
+    String READ_ENTRY_REJECTED = "READ_ENTRY_REJECTED";
     String READ_ENTRY_SCHEDULING_DELAY = "READ_ENTRY_SCHEDULING_DELAY";
     String READ_ENTRY_FENCE_REQUEST = "READ_ENTRY_FENCE_REQUEST";
     String READ_ENTRY_FENCE_WAIT = "READ_ENTRY_FENCE_WAIT";
@@ -101,6 +102,7 @@ public interface BookKeeperServerStats {
     //
 
     String JOURNAL_SCOPE = "journal";
+    String JOURNAL_DIRS = "JOURNAL_DIRS";
     String JOURNAL_ADD_ENTRY = "JOURNAL_ADD_ENTRY";
     String JOURNAL_FORCE_LEDGER = "JOURNAL_FORCE_LEDGER";
     String JOURNAL_SYNC = "JOURNAL_SYNC";
@@ -110,8 +112,10 @@ public interface BookKeeperServerStats {
     String JOURNAL_FORCE_WRITE_LATENCY = "JOURNAL_FORCE_WRITE_LATENCY";
     String JOURNAL_FORCE_WRITE_BATCH_ENTRIES = "JOURNAL_FORCE_WRITE_BATCH_ENTRIES";
     String JOURNAL_FORCE_WRITE_BATCH_BYTES = "JOURNAL_FORCE_WRITE_BATCH_BYTES";
+    String JOURNAL_FORCE_WRITE_GROUPING_FAILURES = "JOURNAL_FORCE_WRITE_GROUPING_FAILURES";
     String JOURNAL_FLUSH_LATENCY = "JOURNAL_FLUSH_LATENCY";
     String JOURNAL_QUEUE_LATENCY = "JOURNAL_QUEUE_LATENCY";
+    String JOURNAL_QUEUE_MAX_SIZE = "JOURNAL_QUEUE_MAX_SIZE";
     String JOURNAL_PROCESS_TIME_LATENCY = "JOURNAL_PROCESS_TIME_LATENCY";
     String JOURNAL_CREATION_LATENCY = "JOURNAL_CREATION_LATENCY";
     String JOURNAL_MEMORY_MAX = "JOURNAL_MEMORY_MAX";
@@ -184,6 +188,7 @@ public interface BookKeeperServerStats {
     String LD_LEDGER_SCOPE = "ledger";
     String LD_INDEX_SCOPE = "index";
     String LD_WRITABLE_DIRS = "writable_dirs";
+    String LD_NUM_DIRS = "num_dirs";
 
     // EntryLogManagerForEntryLogPerLedger Stats
     String ENTRYLOGGER_SCOPE = "entrylogger";
