@@ -43,7 +43,6 @@ import org.apache.bookkeeper.client.api.LedgerMetadata;
 import org.apache.bookkeeper.common.annotation.InterfaceAudience.Private;
 import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.meta.LedgerUnderreplicationManager;
-import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.replication.ReplicationException;
 import org.apache.bookkeeper.tools.cli.commands.autorecovery.ListUnderReplicatedCommand;
 import org.apache.bookkeeper.tools.cli.commands.autorecovery.LostBookieRecoveryDelayCommand;
@@ -51,6 +50,7 @@ import org.apache.bookkeeper.tools.cli.commands.autorecovery.QueryAutoRecoverySt
 import org.apache.bookkeeper.tools.cli.commands.autorecovery.ToggleCommand;
 import org.apache.bookkeeper.tools.cli.commands.autorecovery.TriggerAuditCommand;
 import org.apache.bookkeeper.tools.cli.commands.autorecovery.WhoIsAuditorCommand;
+import org.apache.bookkeeper.tools.cli.commands.bookie.CheckDBLedgersIndexCommand;
 import org.apache.bookkeeper.tools.cli.commands.bookie.ConvertToDBStorageCommand;
 import org.apache.bookkeeper.tools.cli.commands.bookie.ConvertToInterleavedStorageCommand;
 import org.apache.bookkeeper.tools.cli.commands.bookie.FlipBookieIdCommand;
@@ -67,6 +67,7 @@ import org.apache.bookkeeper.tools.cli.commands.bookie.ReadLedgerCommand;
 import org.apache.bookkeeper.tools.cli.commands.bookie.ReadLogCommand;
 import org.apache.bookkeeper.tools.cli.commands.bookie.ReadLogMetadataCommand;
 import org.apache.bookkeeper.tools.cli.commands.bookie.RebuildDBLedgerLocationsIndexCommand;
+import org.apache.bookkeeper.tools.cli.commands.bookie.RebuildDBLedgersIndexCommand;
 import org.apache.bookkeeper.tools.cli.commands.bookie.RegenerateInterleavedStorageIndexFileCommand;
 import org.apache.bookkeeper.tools.cli.commands.bookie.SanityTestCommand;
 import org.apache.bookkeeper.tools.cli.commands.bookie.UpdateBookieInLedgerCommand;
@@ -104,6 +105,7 @@ import org.apache.commons.configuration.CompositeConfiguration;
 import org.apache.commons.configuration.PropertiesConfiguration;
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.lang.StringUtils;
+import org.apache.zookeeper.KeeperException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -154,6 +156,8 @@ public class BookieShell implements Tool {
     static final String CMD_CONVERT_TO_DB_STORAGE = "convert-to-db-storage";
     static final String CMD_CONVERT_TO_INTERLEAVED_STORAGE = "convert-to-interleaved-storage";
     static final String CMD_REBUILD_DB_LEDGER_LOCATIONS_INDEX = "rebuild-db-ledger-locations-index";
+    static final String CMD_REBUILD_DB_LEDGERS_INDEX = "rebuild-db-ledgers-index";
+    static final String CMD_CHECK_DB_LEDGERS_INDEX = "check-db-ledgers-index";
     static final String CMD_REGENERATE_INTERLEAVED_STORAGE_INDEX_FILE = "regenerate-interleaved-storage-index-file";
     static final String CMD_QUERY_AUTORECOVERY_STATUS = "queryrecoverystatus";
 
@@ -218,9 +222,11 @@ public class BookieShell implements Tool {
         abstract int runCmd(CommandLine cmdLine) throws Exception;
 
         String cmdName;
+        Options opts;
 
         MyCommand(String cmdName) {
             this.cmdName = cmdName;
+            opts = getOptionsWithHelp();
         }
 
         @Override
@@ -234,6 +240,10 @@ public class BookieShell implements Tool {
             try {
                 BasicParser parser = new BasicParser();
                 CommandLine cmdLine = parser.parse(getOptions(), args);
+                if (cmdLine.hasOption("help")) {
+                    printUsage();
+                    return 0;
+                }
                 return runCmd(cmdLine);
             } catch (ParseException e) {
                 LOG.error("Error parsing command line arguments : ", e);
@@ -248,13 +258,18 @@ public class BookieShell implements Tool {
             System.err.println(cmdName + ": " + getDescription());
             hf.printHelp(getUsage(), getOptions());
         }
+
+        private Options getOptionsWithHelp() {
+            Options opts = new Options();
+            opts.addOption("h", "help", false, "Show the help");
+            return opts;
+        }
     }
 
     /**
      * Format the bookkeeper metadata present in zookeeper.
      */
     class MetaFormatCmd extends MyCommand {
-        Options opts = new Options();
 
         MetaFormatCmd() {
             super(CMD_METAFORMAT);
@@ -304,7 +319,6 @@ public class BookieShell implements Tool {
      * already existing.
      */
     class InitNewCluster extends MyCommand {
-        Options opts = new Options();
 
         InitNewCluster() {
             super(CMD_INITNEWCLUSTER);
@@ -339,7 +353,6 @@ public class BookieShell implements Tool {
      * Nuke bookkeeper metadata of existing cluster in zookeeper.
      */
     class NukeExistingCluster extends MyCommand {
-        Options opts = new Options();
 
         NukeExistingCluster() {
             super(CMD_NUKEEXISTINGCLUSTER);
@@ -384,7 +397,6 @@ public class BookieShell implements Tool {
      * Formats the local data present in current bookie server.
      */
     class BookieFormatCmd extends MyCommand {
-        Options opts = new Options();
 
         public BookieFormatCmd() {
             super(CMD_BOOKIEFORMAT);
@@ -432,7 +444,6 @@ public class BookieShell implements Tool {
      * indexDirs are empty and there is no registered Bookie with this BookieId.
      */
     class InitBookieCmd extends MyCommand {
-        Options opts = new Options();
 
         public InitBookieCmd() {
             super(CMD_INITBOOKIE);
@@ -466,7 +477,6 @@ public class BookieShell implements Tool {
      * Recover command for ledger data recovery for failed bookie.
      */
     class RecoverCmd extends MyCommand {
-        Options opts = new Options();
 
         public RecoverCmd() {
             super(CMD_RECOVER);
@@ -535,11 +545,10 @@ public class BookieShell implements Tool {
      * Ledger Command Handles ledger related operations.
      */
     class LedgerCmd extends MyCommand {
-        Options lOpts = new Options();
 
         LedgerCmd() {
             super(CMD_LEDGER);
-            lOpts.addOption("m", "meta", false, "Print meta information");
+            opts.addOption("m", "meta", false, "Print meta information");
         }
 
         @Override
@@ -567,7 +576,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
     }
 
@@ -575,22 +584,21 @@ public class BookieShell implements Tool {
      * Command for reading ledger entries.
      */
     class ReadLedgerEntriesCmd extends MyCommand {
-        Options lOpts = new Options();
 
         ReadLedgerEntriesCmd() {
             super(CMD_READ_LEDGER_ENTRIES);
-            lOpts.addOption("m", "msg", false, "Print message body");
-            lOpts.addOption("l", "ledgerid", true, "Ledger ID");
-            lOpts.addOption("fe", "firstentryid", true, "First EntryID");
-            lOpts.addOption("le", "lastentryid", true, "Last EntryID");
-            lOpts.addOption("r", "force-recovery", false,
+            opts.addOption("m", "msg", false, "Print message body");
+            opts.addOption("l", "ledgerid", true, "Ledger ID");
+            opts.addOption("fe", "firstentryid", true, "First EntryID");
+            opts.addOption("le", "lastentryid", true, "Last EntryID");
+            opts.addOption("r", "force-recovery", false,
                 "Ensure the ledger is properly closed before reading");
-            lOpts.addOption("b", "bookie", true, "Only read from a specific bookie");
+            opts.addOption("b", "bookie", true, "Only read from a specific bookie");
         }
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
 
         @Override
@@ -613,7 +621,6 @@ public class BookieShell implements Tool {
 
             boolean printMsg = cmdLine.hasOption("m");
             boolean forceRecovery = cmdLine.hasOption("r");
-            final BookieId bookie;
             String bookieAddress;
             if (cmdLine.hasOption("b")) {
                 // A particular bookie was specified
@@ -640,14 +647,18 @@ public class BookieShell implements Tool {
      * Command for listing underreplicated ledgers.
      */
     class ListUnderreplicatedCmd extends MyCommand {
-        Options opts = new Options();
 
         public ListUnderreplicatedCmd() {
             super(CMD_LISTUNDERREPLICATED);
-            opts.addOption("missingreplica", true, "Bookie Id of missing replica");
-            opts.addOption("excludingmissingreplica", true, "Bookie Id of missing replica to ignore");
-            opts.addOption("printmissingreplica", false, "Whether to print missingreplicas list?");
-            opts.addOption("printreplicationworkerid", false, "Whether to print replicationworkerid?");
+            opts.addOption("mr", "missingreplica", true, "Bookie Id of missing replica");
+            opts.addOption("emr", "excludingmissingreplica", true,
+                "Bookie Id of missing replica to ignore");
+            opts.addOption("pmr", "printmissingreplica", false,
+                "Whether to print missingreplicas list?");
+            opts.addOption("prw", "printreplicationworkerid", false,
+                "Whether to print replicationworkerid?");
+            opts.addOption("c", "onlydisplayledgercount", false,
+                "Only display underreplicated ledger count");
         }
 
         @Override
@@ -674,12 +685,14 @@ public class BookieShell implements Tool {
             final String excludingBookieId = cmdLine.getOptionValue("excludingmissingreplica");
             final boolean printMissingReplica = cmdLine.hasOption("printmissingreplica");
             final boolean printReplicationWorkerId = cmdLine.hasOption("printreplicationworkerid");
+            final boolean onlyDisplayLedgerCount = cmdLine.hasOption("onlydisplayledgercount");
 
             ListUnderReplicatedCommand.LURFlags flags = new ListUnderReplicatedCommand.LURFlags()
                                                             .missingReplica(includingBookieId)
                                                             .excludingMissingReplica(excludingBookieId)
                                                             .printMissingReplica(printMissingReplica)
-                                                            .printReplicationWorkerId(printReplicationWorkerId);
+                                                            .printReplicationWorkerId(printReplicationWorkerId)
+                                                            .onlyDisplayLedgerCount(onlyDisplayLedgerCount);
             ListUnderReplicatedCommand cmd = new ListUnderReplicatedCommand(ledgerIdFormatter);
             cmd.apply(bkConf, flags);
             return 0;
@@ -692,12 +705,11 @@ public class BookieShell implements Tool {
      * Command to list all ledgers in the cluster.
      */
     class ListLedgersCmd extends MyCommand {
-        Options lOpts = new Options();
 
         ListLedgersCmd() {
             super(CMD_LISTLEDGERS);
-            lOpts.addOption("m", "meta", false, "Print metadata");
-            lOpts.addOption("bookieid", true, "List ledgers residing in this bookie");
+            opts.addOption("m", "meta", false, "Print metadata");
+            opts.addOption("bookieid", true, "List ledgers residing in this bookie");
         }
 
         @Override
@@ -725,7 +737,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
     }
 
@@ -733,12 +745,11 @@ public class BookieShell implements Tool {
      * List active ledgers on entry log file.
      **/
     class ListActiveLedgersCmd extends MyCommand {
-        Options lOpts = new Options();
 
         ListActiveLedgersCmd() {
             super(CMD_ACTIVE_LEDGERS_ON_ENTRY_LOG_FILE);
-            lOpts.addOption("l", "logId", true, "Entry log file id");
-            lOpts.addOption("t", "timeout", true, "Read timeout(ms)");
+            opts.addOption("l", "logId", true, "Entry log file id");
+            opts.addOption("t", "timeout", true, "Read timeout(ms)");
         }
 
         @Override
@@ -772,7 +783,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
     }
 
@@ -787,13 +798,12 @@ public class BookieShell implements Tool {
      * Print the metadata for a ledger.
      */
     class LedgerMetadataCmd extends MyCommand {
-        Options lOpts = new Options();
 
         LedgerMetadataCmd() {
             super(CMD_LEDGERMETADATA);
-            lOpts.addOption("l", "ledgerid", true, "Ledger ID");
-            lOpts.addOption("dumptofile", true, "Dump metadata for ledger, to a file");
-            lOpts.addOption("restorefromfile", true, "Restore metadata for ledger, from a file");
+            opts.addOption("l", "ledgerid", true, "Ledger ID");
+            opts.addOption("dumptofile", true, "Dump metadata for ledger, to a file");
+            opts.addOption("restorefromfile", true, "Restore metadata for ledger, from a file");
         }
 
         @Override
@@ -834,7 +844,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
     }
 
@@ -842,7 +852,6 @@ public class BookieShell implements Tool {
      * Check local storage for inconsistencies.
      */
     class LocalConsistencyCheck extends MyCommand {
-        Options lOpts = new Options();
 
         LocalConsistencyCheck() {
             super(CMD_LOCALCONSISTENCYCHECK);
@@ -867,7 +876,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
     }
 
@@ -875,14 +884,13 @@ public class BookieShell implements Tool {
      * Simple test to create a ledger and write to it.
      */
     class SimpleTestCmd extends MyCommand {
-        Options lOpts = new Options();
 
         SimpleTestCmd() {
             super(CMD_SIMPLETEST);
-            lOpts.addOption("e", "ensemble", true, "Ensemble size (default 3)");
-            lOpts.addOption("w", "writeQuorum", true, "Write quorum size (default 2)");
-            lOpts.addOption("a", "ackQuorum", true, "Ack quorum size (default 2)");
-            lOpts.addOption("n", "numEntries", true, "Entries to write (default 1000)");
+            opts.addOption("e", "ensemble", true, "Ensemble size (default 3)");
+            opts.addOption("w", "writeQuorum", true, "Write quorum size (default 2)");
+            opts.addOption("a", "ackQuorum", true, "Ack quorum size (default 2)");
+            opts.addOption("n", "numEntries", true, "Entries to write (default 1000)");
         }
 
         @Override
@@ -916,7 +924,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
     }
 
@@ -924,17 +932,16 @@ public class BookieShell implements Tool {
      * Command to run a bookie sanity test.
      */
     class BookieSanityTestCmd extends MyCommand {
-        Options lOpts = new Options();
 
         BookieSanityTestCmd() {
             super(CMD_BOOKIESANITYTEST);
-            lOpts.addOption("e", "entries", true, "Total entries to be added for the test (default 10)");
-            lOpts.addOption("t", "timeout", true, "Timeout for write/read operations in seconds (default 1)");
+            opts.addOption("e", "entries", true, "Total entries to be added for the test (default 10)");
+            opts.addOption("t", "timeout", true, "Timeout for write/read operations in seconds (default 1)");
         }
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
 
         @Override
@@ -960,15 +967,14 @@ public class BookieShell implements Tool {
      * Command to read entry log files.
      */
     class ReadLogCmd extends MyCommand {
-        Options rlOpts = new Options();
 
         ReadLogCmd() {
             super(CMD_READLOG);
-            rlOpts.addOption("m", "msg", false, "Print message body");
-            rlOpts.addOption("l", "ledgerid", true, "Ledger ID");
-            rlOpts.addOption("e", "entryid", true, "Entry ID");
-            rlOpts.addOption("sp", "startpos", true, "Start Position");
-            rlOpts.addOption("ep", "endpos", true, "End Position");
+            opts.addOption("m", "msg", false, "Print message body");
+            opts.addOption("l", "ledgerid", true, "Ledger ID");
+            opts.addOption("e", "entryid", true, "Entry ID");
+            opts.addOption("sp", "startpos", true, "Start Position");
+            opts.addOption("ep", "endpos", true, "End Position");
         }
 
         @Override
@@ -1020,7 +1026,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return rlOpts;
+            return opts;
         }
     }
 
@@ -1028,7 +1034,6 @@ public class BookieShell implements Tool {
      * Command to print metadata of entrylog.
      */
     class ReadLogMetadataCmd extends MyCommand {
-        Options rlOpts = new Options();
 
         ReadLogMetadataCmd() {
             super(CMD_READLOGMETADATA);
@@ -1069,7 +1074,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return rlOpts;
+            return opts;
         }
     }
 
@@ -1077,12 +1082,11 @@ public class BookieShell implements Tool {
      * Command to read journal files.
      */
     class ReadJournalCmd extends MyCommand {
-        Options rjOpts = new Options();
 
         ReadJournalCmd() {
             super(CMD_READJOURNAL);
-            rjOpts.addOption("dir", true, "Journal directory (needed if more than one journal configured)");
-            rjOpts.addOption("m", "msg", false, "Print message body");
+            opts.addOption("dir", true, "Journal directory (needed if more than one journal configured)");
+            opts.addOption("m", "msg", false, "Print message body");
         }
 
         @Override
@@ -1127,7 +1131,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return rjOpts;
+            return opts;
         }
     }
 
@@ -1158,7 +1162,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return new Options();
+            return opts;
         }
     }
 
@@ -1166,7 +1170,6 @@ public class BookieShell implements Tool {
      * List available bookies.
      */
     class ListBookiesCmd extends MyCommand {
-        Options opts = new Options();
 
         ListBookiesCmd() {
             super(CMD_LISTBOOKIES);
@@ -1228,7 +1231,6 @@ public class BookieShell implements Tool {
     }
 
     class ListDiskFilesCmd extends MyCommand {
-        Options opts = new Options();
 
         ListDiskFilesCmd() {
             super(CMD_LISTFILESONDISC);
@@ -1305,7 +1307,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return new Options();
+            return opts;
         }
     }
 
@@ -1313,7 +1315,6 @@ public class BookieShell implements Tool {
      * Command for administration of autorecovery.
      */
     class AutoRecoveryCmd extends MyCommand {
-        Options opts = new Options();
 
         public AutoRecoveryCmd() {
             super(CMD_AUTORECOVERY);
@@ -1357,7 +1358,6 @@ public class BookieShell implements Tool {
      * Command to query autorecovery status.
      */
     class QueryAutoRecoveryStatusCmd extends MyCommand {
-        Options opts = new Options();
 
         public QueryAutoRecoveryStatusCmd() {
             super(CMD_QUERY_AUTORECOVERY_STATUS);
@@ -1393,7 +1393,6 @@ public class BookieShell implements Tool {
      * Setter and Getter for LostBookieRecoveryDelay value (in seconds) in metadata store.
      */
     class LostBookieRecoveryDelayCmd extends MyCommand {
-        Options opts = new Options();
 
         public LostBookieRecoveryDelayCmd() {
             super(CMD_LOSTBOOKIERECOVERYDELAY);
@@ -1437,7 +1436,6 @@ public class BookieShell implements Tool {
      * Print which node has the auditor lock.
      */
     class WhoIsAuditorCmd extends MyCommand {
-        Options opts = new Options();
 
         public WhoIsAuditorCmd() {
             super(CMD_WHOISAUDITOR);
@@ -1471,7 +1469,6 @@ public class BookieShell implements Tool {
      * Prints the instanceid of the cluster.
      */
     class WhatIsInstanceId extends MyCommand {
-        Options opts = new Options();
 
         public WhatIsInstanceId() {
             super(CMD_WHATISINSTANCEID);
@@ -1504,7 +1501,6 @@ public class BookieShell implements Tool {
      * Update cookie command.
      */
     class UpdateCookieCmd extends MyCommand {
-        Options opts = new Options();
         private static final String BOOKIEID = "bookieId";
         private static final String EXPANDSTORAGE = "expandstorage";
         private static final String LIST = "list";
@@ -1591,7 +1587,6 @@ public class BookieShell implements Tool {
      * Update ledger command.
      */
     class UpdateLedgerCmd extends MyCommand {
-        private final Options opts = new Options();
 
         UpdateLedgerCmd() {
             super(CMD_UPDATELEDGER);
@@ -1667,7 +1662,6 @@ public class BookieShell implements Tool {
      * Update bookie into ledger command.
      */
     class UpdateBookieInLedgerCmd extends MyCommand {
-        private final Options opts = new Options();
 
         UpdateBookieInLedgerCmd() {
             super(CMD_UPDATE_BOOKIE_IN_LEDGER);
@@ -1746,12 +1740,11 @@ public class BookieShell implements Tool {
      * Command to delete a given ledger.
      */
     class DeleteLedgerCmd extends MyCommand {
-        Options lOpts = new Options();
 
         DeleteLedgerCmd() {
             super(CMD_DELETELEDGER);
-            lOpts.addOption("l", "ledgerid", true, "Ledger ID");
-            lOpts.addOption("f", "force", false, "Whether to force delete the Ledger without prompt..?");
+            opts.addOption("l", "ledgerid", true, "Ledger ID");
+            opts.addOption("f", "force", false, "Whether to force delete the Ledger without prompt..?");
         }
 
         @Override
@@ -1779,7 +1772,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
     }
 
@@ -1788,7 +1781,6 @@ public class BookieShell implements Tool {
      * the bookies in the cluster.
      */
     class BookieInfoCmd extends MyCommand {
-        Options lOpts = new Options();
 
         BookieInfoCmd() {
             super(CMD_BOOKIEINFO);
@@ -1806,7 +1798,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
 
         @Override
@@ -1821,7 +1813,6 @@ public class BookieShell implements Tool {
      * Command to trigger AuditTask by resetting lostBookieRecoveryDelay to its current value.
      */
     class TriggerAuditCmd extends MyCommand {
-        Options opts = new Options();
 
         TriggerAuditCmd() {
             super(CMD_TRIGGERAUDIT);
@@ -1851,7 +1842,6 @@ public class BookieShell implements Tool {
     }
 
     class ForceAuditorChecksCmd extends MyCommand {
-        Options opts = new Options();
 
         ForceAuditorChecksCmd() {
             super(CMD_FORCEAUDITCHECKS);
@@ -1906,7 +1896,7 @@ public class BookieShell implements Tool {
                                 underreplicationManager.setReplicasCheckCTime(time);
                             }
                         }
-                    } catch (InterruptedException | ReplicationException e) {
+                    } catch (InterruptedException | KeeperException | ReplicationException e) {
                         LOG.error("Exception while trying to reset last run time ", e);
                         return -1;
                     }
@@ -1927,11 +1917,10 @@ public class BookieShell implements Tool {
      * server.
      */
     class DecommissionBookieCmd extends MyCommand {
-        Options lOpts = new Options();
 
         DecommissionBookieCmd() {
             super(CMD_DECOMMISSIONBOOKIE);
-            lOpts.addOption("bookieid", true, "decommission a remote bookie");
+            opts.addOption("bookieid", true, "decommission a remote bookie");
         }
 
         @Override
@@ -1947,7 +1936,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
 
         @Override
@@ -1965,11 +1954,10 @@ public class BookieShell implements Tool {
      * Command to retrieve remote bookie endpoint information.
      */
     class EndpointInfoCmd extends MyCommand {
-        Options lOpts = new Options();
 
         EndpointInfoCmd() {
             super(CMD_ENDPOINTINFO);
-            lOpts.addOption("b", "bookieid", true, "Bookie Id");
+            opts.addOption("b", "bookieid", true, "Bookie Id");
         }
 
         @Override
@@ -1984,7 +1972,7 @@ public class BookieShell implements Tool {
 
         @Override
         Options getOptions() {
-            return lOpts;
+            return opts;
         }
 
         @Override
@@ -2015,7 +2003,6 @@ public class BookieShell implements Tool {
      * Convert bookie indexes from InterleavedStorage to DbLedgerStorage format.
      */
     class ConvertToDbStorageCmd extends MyCommand {
-        Options opts = new Options();
 
         public ConvertToDbStorageCmd() {
             super(CMD_CONVERT_TO_DB_STORAGE);
@@ -2050,7 +2037,6 @@ public class BookieShell implements Tool {
      * Convert bookie indexes from DbLedgerStorage to InterleavedStorage format.
      */
     class ConvertToInterleavedStorageCmd extends MyCommand {
-        Options opts = new Options();
 
         public ConvertToInterleavedStorageCmd() {
             super(CMD_CONVERT_TO_INTERLEAVED_STORAGE);
@@ -2084,7 +2070,6 @@ public class BookieShell implements Tool {
      * Rebuild DbLedgerStorage locations index.
      */
     class RebuildDbLedgerLocationsIndexCmd extends MyCommand {
-        Options opts = new Options();
 
         public RebuildDbLedgerLocationsIndexCmd() {
             super(CMD_REBUILD_DB_LEDGER_LOCATIONS_INDEX);
@@ -2113,11 +2098,89 @@ public class BookieShell implements Tool {
         }
     }
 
+    /**
+     * Rebuild DbLedgerStorage ledgers index.
+     */
+    class RebuildDbLedgersIndexCmd extends MyCommand {
+
+        public RebuildDbLedgersIndexCmd() {
+            super(CMD_REBUILD_DB_LEDGERS_INDEX);
+            opts.addOption("v", "verbose", false, "Verbose logging, print the ledgers added to the new index");
+        }
+
+        @Override
+        Options getOptions() {
+            return opts;
+        }
+
+        @Override
+        String getDescription() {
+            return "Rebuild DbLedgerStorage ledgers index by scanning "
+                + "the journal and entry logs (sets all ledgers to fenced)";
+        }
+
+        @Override
+        String getUsage() {
+            return CMD_REBUILD_DB_LEDGERS_INDEX;
+        }
+
+        @Override
+        int runCmd(CommandLine cmdLine) throws Exception {
+            RebuildDBLedgersIndexCommand.RebuildLedgersIndexFlags flags =
+                    new RebuildDBLedgersIndexCommand.RebuildLedgersIndexFlags();
+            flags.verbose(cmdLine.hasOption("v"));
+            RebuildDBLedgersIndexCommand cmd = new RebuildDBLedgersIndexCommand();
+            if (cmd.apply(bkConf, flags)) {
+                return 0;
+            } else {
+                return -1;
+            }
+        }
+    }
+
+    /**
+     * Rebuild DbLedgerStorage ledgers index.
+     */
+    class CheckDbLedgersIndexCmd extends MyCommand {
+
+        public CheckDbLedgersIndexCmd() {
+            super(CMD_CHECK_DB_LEDGERS_INDEX);
+            opts.addOption("v", "verbose", false, "Verbose logging, print the ledger data in the index.");
+        }
+
+        @Override
+        Options getOptions() {
+            return opts;
+        }
+
+        @Override
+        String getDescription() {
+            return "Check DbLedgerStorage ledgers index by performing a read scan";
+        }
+
+        @Override
+        String getUsage() {
+            return CMD_CHECK_DB_LEDGERS_INDEX;
+        }
+
+        @Override
+        int runCmd(CommandLine cmdLine) throws Exception {
+            CheckDBLedgersIndexCommand.CheckLedgersIndexFlags flags =
+                    new CheckDBLedgersIndexCommand.CheckLedgersIndexFlags();
+            flags.verbose(cmdLine.hasOption("v"));
+            CheckDBLedgersIndexCommand cmd = new CheckDBLedgersIndexCommand();
+            if (cmd.apply(bkConf, flags)) {
+                return 0;
+            } else {
+                return -1;
+            }
+        }
+    }
+
     /**
      * Regenerate an index file for interleaved storage.
      */
     class RegenerateInterleavedStorageIndexFile extends MyCommand {
-        Options opts = new Options();
 
         public RegenerateInterleavedStorageIndexFile() {
             super(CMD_REGENERATE_INTERLEAVED_STORAGE_INDEX_FILE);
@@ -2211,6 +2274,8 @@ public class BookieShell implements Tool {
         commands.put(CMD_CONVERT_TO_DB_STORAGE, new ConvertToDbStorageCmd());
         commands.put(CMD_CONVERT_TO_INTERLEAVED_STORAGE, new ConvertToInterleavedStorageCmd());
         commands.put(CMD_REBUILD_DB_LEDGER_LOCATIONS_INDEX, new RebuildDbLedgerLocationsIndexCmd());
+        commands.put(CMD_REBUILD_DB_LEDGERS_INDEX, new RebuildDbLedgersIndexCmd());
+        commands.put(CMD_CHECK_DB_LEDGERS_INDEX, new CheckDbLedgersIndexCmd());
         commands.put(CMD_REGENERATE_INTERLEAVED_STORAGE_INDEX_FILE, new RegenerateInterleavedStorageIndexFile());
         commands.put(CMD_HELP, new HelpCmd());
         commands.put(CMD_LOSTBOOKIERECOVERYDELAY, new LostBookieRecoveryDelayCmd());
@@ -2232,12 +2297,12 @@ public class BookieShell implements Tool {
     @Override
     public void setConf(CompositeConfiguration conf) throws Exception {
         bkConf.loadConf(conf);
-        journalDirectories = Bookie.getCurrentDirectories(bkConf.getJournalDirs());
-        ledgerDirectories = Bookie.getCurrentDirectories(bkConf.getLedgerDirs());
+        journalDirectories = BookieImpl.getCurrentDirectories(bkConf.getJournalDirs());
+        ledgerDirectories = BookieImpl.getCurrentDirectories(bkConf.getLedgerDirs());
         if (null == bkConf.getIndexDirs()) {
             indexDirectories = ledgerDirectories;
         } else {
-            indexDirectories = Bookie.getCurrentDirectories(bkConf.getIndexDirs());
+            indexDirectories = BookieImpl.getCurrentDirectories(bkConf.getIndexDirs());
         }
         pageSize = bkConf.getPageSize();
         entriesPerPage = pageSize / 8;
@@ -2464,4 +2529,5 @@ public class BookieShell implements Tool {
     private static boolean getOptionalValue(String optValue, String optName) {
         return StringUtils.equals(optValue, optName);
     }
+
 }
