@@ -32,7 +32,6 @@ import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufAllocator;
 import io.netty.buffer.PooledByteBufAllocator;
 import io.netty.buffer.Unpooled;
-import io.netty.util.ReferenceCountUtil;
 import io.netty.util.concurrent.FastThreadLocal;
 import java.io.BufferedReader;
 import java.io.File;
@@ -189,7 +188,7 @@ public class EntryLogger {
                     throw e;
                 }
             } finally {
-                ReferenceCountUtil.release(serializedMap);
+                serializedMap.release();
             }
             // Flush the ledger's map out before we write the header.
             // Otherwise the header might point to something that is not fully
@@ -483,6 +482,27 @@ public class EntryLogger {
         return recentlyCreatedEntryLogsStatus.getLeastUnflushedLogId();
     }
 
+    /**
+     * Get the last log id created so far. If entryLogPerLedger is enabled, the Garbage Collector
+     * process needs to look beyond the least unflushed entry log file, as there may be entry logs
+     * ready to be garbage collected.
+     *
+     * @return last entry log id created.
+     */
+    long getLastLogId() {
+        return recentlyCreatedEntryLogsStatus.getLastLogId();
+    }
+
+    /**
+     * Returns whether the current log id exists and has been rotated already.
+     *
+     * @param entryLogId EntryLog id to check.
+     * @return Whether the given entryLogId exists and has been rotated.
+     */
+    boolean isFlushedEntryLog(Long entryLogId) {
+        return recentlyCreatedEntryLogsStatus.isFlushedEntryLog(entryLogId);
+    }
+
     long getPreviousAllocatedEntryLogId() {
         return entryLoggerAllocator.getPreallocatedLogId();
     }
@@ -830,18 +850,27 @@ public class EntryLogger {
             if (validateEntry) {
                 validateEntry(ledgerId, entryId, entryLogId, pos, sizeBuff);
             }
+        } catch (EntryLookupException.MissingEntryException entryLookupError) {
+            throw new Bookie.NoEntryException("Short read from entrylog " + entryLogId,
+                    ledgerId, entryId);
         } catch (EntryLookupException e) {
-            throw new IOException("Bad entry read from log file id: " + entryLogId, e);
+            throw new IOException(e.toString());
         }
 
         ByteBuf data = allocator.buffer(entrySize, entrySize);
         int rc = readFromLogChannel(entryLogId, fc, data, pos);
         if (rc != entrySize) {
-            ReferenceCountUtil.release(data);
-            throw new IOException("Bad entry read from log file id: " + entryLogId,
-                    new EntryLookupException("Short read for " + ledgerId + "@"
+            // Note that throwing NoEntryException here instead of IOException is not
+            // without risk. If all bookies in a quorum throw this same exception
+            // the client will assume that it has reached the end of the ledger.
+            // However, this may not be the case, as a very specific error condition
+            // could have occurred, where the length of the entry was corrupted on all
+            // replicas. However, the chance of this happening is very very low, so
+            // returning NoEntryException is mostly safe.
+            data.release();
+            throw new Bookie.NoEntryException("Short read for " + ledgerId + "@"
                                               + entryId + " in " + entryLogId + "@"
-                                              + pos + "(" + rc + "!=" + entrySize + ")"));
+                                              + pos + "(" + rc + "!=" + entrySize + ")", ledgerId, entryId);
         }
         data.writerIndex(entrySize);
 
@@ -875,7 +904,7 @@ public class EntryLogger {
             int ledgersCount = headers.readInt();
             return new Header(headerVersion, ledgersMapOffset, ledgersCount);
         } finally {
-            ReferenceCountUtil.release(headers);
+            headers.release();
         }
     }
 
@@ -992,11 +1021,6 @@ public class EntryLogger {
                 long offset = pos;
                 pos += 4;
                 int entrySize = headerBuffer.readInt();
-                if (entrySize <= 0) { // hitting padding
-                    pos++;
-                    headerBuffer.clear();
-                    continue;
-                }
                 long ledgerId = headerBuffer.readLong();
                 headerBuffer.clear();
 
@@ -1007,6 +1031,11 @@ public class EntryLogger {
                 }
                 // read the entry
                 data.clear();
+                if (entrySize <= 0) {
+                    LOG.warn("bad read for ledger entry from entryLog {}@{} (entry size {})",
+                            entryLogId, pos, entrySize);
+                    return;
+                }
                 data.capacity(entrySize);
                 int rc = readFromLogChannel(entryLogId, bc, data, pos);
                 if (rc != entrySize) {
@@ -1021,12 +1050,12 @@ public class EntryLogger {
                 pos += entrySize;
             }
         } finally {
-            ReferenceCountUtil.release(data);
+            data.release();
         }
     }
 
     public EntryLogMetadata getEntryLogMetadata(long entryLogId) throws IOException {
-        // First try to extract the EntryLogMetada from the index, if there's no index then fallback to scanning the
+        // First try to extract the EntryLogMetadata from the index, if there's no index then fallback to scanning the
         // entry log
         try {
             return extractEntryLogMetadataFromIndex(entryLogId);
@@ -1038,6 +1067,20 @@ public class EntryLogger {
         }
     }
 
+    public EntryLogMetadata getEntryLogMetadata(long entryLogId, AbstractLogCompactor.Throttler throttler)
+        throws IOException {
+        // First try to extract the EntryLogMetadata from the index, if there's no index then fallback to scanning the
+        // entry log
+        try {
+            return extractEntryLogMetadataFromIndex(entryLogId);
+        } catch (Exception e) {
+            LOG.info("Failed to get ledgers map index from: {}.log : {}", entryLogId, e.getMessage());
+
+            // Fall-back to scanning
+            return extractEntryLogMetadataByScanning(entryLogId, throttler);
+        }
+    }
+
     EntryLogMetadata extractEntryLogMetadataFromIndex(long entryLogId) throws IOException {
         Header header = getHeaderForLogId(entryLogId);
 
@@ -1110,7 +1153,7 @@ public class EntryLogger {
         } catch (IndexOutOfBoundsException e) {
             throw new IOException(e);
         } finally {
-            ReferenceCountUtil.release(ledgersMap);
+            ledgersMap.release();
         }
 
         if (meta.getLedgersMap().size() != header.ledgersCount) {
@@ -1122,12 +1165,21 @@ public class EntryLogger {
     }
 
     private EntryLogMetadata extractEntryLogMetadataByScanning(long entryLogId) throws IOException {
+        return extractEntryLogMetadataByScanning(entryLogId, null);
+    }
+
+    private EntryLogMetadata extractEntryLogMetadataByScanning(long entryLogId,
+                                                               AbstractLogCompactor.Throttler throttler)
+        throws IOException {
         final EntryLogMetadata meta = new EntryLogMetadata(entryLogId);
 
         // Read through the entry log file and extract the entry log meta
         scanEntryLog(entryLogId, new EntryLogScanner() {
             @Override
             public void process(long ledgerId, long offset, ByteBuf entry) throws IOException {
+                if (throttler != null) {
+                    throttler.acquire(entry.readableBytes());
+                }
                 // add new entry size of a ledger to entry log meta
                 meta.addLedgerSize(ledgerId, entry.readableBytes() + 4);
             }
@@ -1241,5 +1293,14 @@ public class EntryLogger {
         synchronized long getLeastUnflushedLogId() {
             return leastUnflushedLogId;
         }
+
+        synchronized long getLastLogId() {
+            return !entryLogsStatusMap.isEmpty() ? entryLogsStatusMap.lastKey() : 0;
+        }
+
+        synchronized boolean isFlushedEntryLog(Long entryLogId) {
+            return entryLogsStatusMap.containsKey(entryLogId) && entryLogsStatusMap.get(entryLogId)
+                    || entryLogId < leastUnflushedLogId;
+        }
     }
 }
