@@ -31,7 +31,6 @@ import io.netty.buffer.Unpooled;
 import io.netty.buffer.UnpooledByteBufAllocator;
 import io.netty.util.Recycler;
 import io.netty.util.Recycler.Handle;
-import io.netty.util.concurrent.DefaultThreadFactory;
 
 import java.io.File;
 import java.io.FileInputStream;
@@ -46,6 +45,7 @@ import java.util.concurrent.ArrayBlockingQueue;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
+import java.util.concurrent.ThreadFactory;
 import java.util.concurrent.TimeUnit;
 
 import org.apache.bookkeeper.bookie.LedgerDirsManager.NoWritableLedgerDirException;
@@ -60,6 +60,7 @@ import org.apache.bookkeeper.stats.Counter;
 import org.apache.bookkeeper.stats.NullStatsLogger;
 import org.apache.bookkeeper.stats.OpStatsLogger;
 import org.apache.bookkeeper.stats.StatsLogger;
+import org.apache.bookkeeper.stats.ThreadRegistry;
 import org.apache.bookkeeper.util.IOUtils;
 import org.apache.bookkeeper.util.MathUtils;
 import org.slf4j.Logger;
@@ -74,7 +75,6 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
 
     private static final RecyclableArrayList.Recycler<QueueEntry> entryListRecycler =
         new RecyclableArrayList.Recycler<QueueEntry>();
-    private static final RecyclableArrayList<QueueEntry> EMPTY_ARRAY_LIST = new RecyclableArrayList<>();
 
     /**
      * Filter to pickup journals.
@@ -103,7 +103,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
      * @param filter journal id filter
      * @return list of filtered ids
      */
-    static List<Long> listJournalIds(File journalDir, JournalIdFilter filter) {
+    public static List<Long> listJournalIds(File journalDir, JournalIdFilter filter) {
         File[] logFiles = journalDir.listFiles();
         if (logFiles == null || logFiles.length == 0) {
             return Collections.emptyList();
@@ -229,7 +229,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
          * The last mark should first be max journal log id,
          * and then max log position in max journal log.
          */
-        public void readLog() {
+        void readLog() {
             byte[] buff = new byte[16];
             ByteBuffer bb = ByteBuffer.wrap(buff);
             LogMark mark = new LogMark();
@@ -307,11 +307,11 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
 
         OpStatsLogger journalAddEntryStats;
         Counter journalCbQueueSize;
-
+        Counter callbackTime;
 
         static QueueEntry create(ByteBuf entry, boolean ackBeforeSync, long ledgerId, long entryId,
                 WriteCallback cb, Object ctx, long enqueueTime, OpStatsLogger journalAddEntryStats,
-                Counter journalCbQueueSize) {
+                Counter journalCbQueueSize, Counter callbackTime) {
             QueueEntry qe = RECYCLER.get();
             qe.entry = entry;
             qe.ackBeforeSync = ackBeforeSync;
@@ -322,11 +322,13 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
             qe.enqueueTime = enqueueTime;
             qe.journalAddEntryStats = journalAddEntryStats;
             qe.journalCbQueueSize = journalCbQueueSize;
+            qe.callbackTime = callbackTime;
             return qe;
         }
 
         @Override
         public void run() {
+            long startTime = System.nanoTime();
             if (LOG.isDebugEnabled()) {
                 LOG.debug("Acknowledge Ledger: {}, Entry: {}", ledgerId, entryId);
             }
@@ -334,6 +336,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
             journalAddEntryStats.registerSuccessfulEvent(MathUtils.elapsedNanos(enqueueTime), TimeUnit.NANOSECONDS);
             cb.writeComplete(0, ledgerId, entryId, null, ctx);
             recycle();
+            callbackTime.add(MathUtils.elapsedNanos(startTime));
         }
 
         private final Handle<QueueEntry> recyclerHandle;
@@ -350,11 +353,6 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         };
 
         private void recycle() {
-            this.entry = null;
-            this.cb = null;
-            this.ctx = null;
-            this.journalAddEntryStats = null;
-            this.journalCbQueueSize = null;
             recyclerHandle.recycle(this);
         }
     }
@@ -381,9 +379,9 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                 return 0;
             }
 
+            long startTime = MathUtils.nowInNano();
             try {
                 if (shouldForceWrite) {
-                    long startTime = MathUtils.nowInNano();
                     this.logFile.forceWrite(false);
                     journalStats.getJournalSyncStats()
                         .registerSuccessfulEvent(MathUtils.elapsedNanos(startTime), TimeUnit.NANOSECONDS);
@@ -399,6 +397,10 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                 }
 
                 return forceWriteWaiters.size();
+            } catch (IOException e) {
+                journalStats.getJournalSyncStats()
+                        .registerFailedEvent(MathUtils.elapsedNanos(startTime), TimeUnit.NANOSECONDS);
+                throw e;
             } finally {
                 closeFileIfNecessary();
             }
@@ -472,15 +474,19 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         Thread threadToNotifyOnEx;
         // should we group force writes
         private final boolean enableGroupForceWrites;
-        // make flush interval as a parameter
-        public ForceWriteThread(Thread threadToNotifyOnEx, boolean enableGroupForceWrites) {
+        private final Counter forceWriteThreadTime;
+        public ForceWriteThread(Thread threadToNotifyOnEx,
+                                boolean enableGroupForceWrites,
+                                StatsLogger statsLogger) {
             super("ForceWriteThread");
             this.threadToNotifyOnEx = threadToNotifyOnEx;
             this.enableGroupForceWrites = enableGroupForceWrites;
+            this.forceWriteThreadTime = statsLogger.getThreadScopedCounter("force-write-thread-time");
         }
         @Override
         public void run() {
             LOG.info("ForceWrite Thread started");
+            ThreadRegistry.register(super.getName(), 0);
 
             if (conf.isBusyWaitEnabled()) {
                 try {
@@ -492,10 +498,14 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
 
             boolean shouldForceWrite = true;
             int numReqInLastForceWrite = 0;
+            long busyStartTime = System.nanoTime();
             while (running) {
                 ForceWriteRequest req = null;
+                boolean forceWriteMarkerSent = false;
                 try {
+                    forceWriteThreadTime.add(MathUtils.elapsedNanos(busyStartTime));
                     req = forceWriteRequests.take();
+                    busyStartTime = System.nanoTime();
                     // Force write the file and then notify the write completions
                     //
                     if (!req.isMarker) {
@@ -504,7 +514,19 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                             // queue will benefit from this force write - post a marker prior to issuing
                             // the flush so until this marker is encountered we can skip the force write
                             if (enableGroupForceWrites) {
-                                forceWriteRequests.put(createForceWriteRequest(req.logFile, 0, 0, null, false, true));
+                                ForceWriteRequest marker =
+                                    createForceWriteRequest(req.logFile, 0, 0, null, false, true);
+                                forceWriteMarkerSent = forceWriteRequests.offer(marker);
+                                if (!forceWriteMarkerSent) {
+                                    marker.recycle();
+                                    Counter failures = journalStats.getForceWriteGroupingFailures();
+                                    failures.inc();
+                                    LOG.error(
+                                        "Fail to send force write grouping marker,"
+                                        + " Journal.forceWriteRequests queue(capacity {}) is full,"
+                                        + " current failure counter is {}.",
+                                        conf.getJournalQueueSize(), failures.get());
+                                }
                             }
 
                             // If we are about to issue a write, record the number of requests in
@@ -522,6 +544,11 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                     if (enableGroupForceWrites
                             // if its a marker we should switch back to flushing
                             && !req.isMarker
+                            // If group marker sending failed, we can't figure out which writes are
+                            // grouped in this force write. So, abandon it even if other writes could
+                            // be grouped. This should be extremely rare as, usually, queue size is
+                            // large enough to accommodate high flush frequencies.
+                            && forceWriteMarkerSent
                             // This indicates that this is the last request in a given file
                             // so subsequent requests will go to a different file so we should
                             // flush on the next request
@@ -561,6 +588,18 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         }
     }
 
+    private static class CbThreadFactory implements ThreadFactory {
+        private int counter = 0;
+        private String threadBaseName = "bookie-journal-callback";
+
+        public Thread newThread(Runnable r) {
+            int threadOrdinal = counter++;
+            Thread t = new Thread(r, threadBaseName + "-" + threadOrdinal);
+            ThreadRegistry.register(threadBaseName, threadOrdinal, t.getId());
+            return t;
+        }
+    }
+
     static final int PADDING_MASK = -0x100;
 
     static void writePaddingBytes(JournalChannel jc, ByteBuf paddingBuffer, int journalAlignSize)
@@ -601,6 +640,8 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
     final File journalDirectory;
     final ServerConfiguration conf;
     final ForceWriteThread forceWriteThread;
+    final FileChannelProvider fileChannelProvider;
+
     // Time after which we will stop grouping and issue the flush
     private final long maxGroupWaitInNanos;
     // Threshold after which we flush any buffered journal entries
@@ -630,6 +671,10 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
      */
     private final ExecutorService cbThreadPool;
 
+    private final Counter callbackTime;
+    private final Counter journalTime;
+    private static String journalThreadName = "BookieJournal";
+
     // journal entry queue to commit
     final BlockingQueue<QueueEntry> queue;
     final BlockingQueue<ForceWriteRequest> forceWriteRequests;
@@ -650,9 +695,11 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
 
     public Journal(int journalIndex, File journalDirectory, ServerConfiguration conf,
             LedgerDirsManager ledgerDirsManager, StatsLogger statsLogger, ByteBufAllocator allocator) {
-        super("BookieJournal-" + conf.getBookiePort());
+        super(journalThreadName + "-" + conf.getBookiePort());
         this.allocator = allocator;
 
+        StatsLogger journalStatsLogger = statsLogger.scopeLabel("journalIndex", String.valueOf(journalIndex));
+
         if (conf.isBusyWaitEnabled()) {
             // To achieve lower latency, use busy-wait blocking queue implementation
             queue = new BlockingMpscQueue<>(conf.getJournalQueueSize());
@@ -673,7 +720,8 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         this.journalWriteBufferSize = conf.getJournalWriteBufferSizeKB() * KB;
         this.syncData = conf.getJournalSyncData();
         this.maxBackupJournals = conf.getMaxBackupJournals();
-        this.forceWriteThread = new ForceWriteThread(this, conf.getJournalAdaptiveGroupWrites());
+        this.forceWriteThread = new ForceWriteThread(this, conf.getJournalAdaptiveGroupWrites(),
+                journalStatsLogger);
         this.maxGroupWaitInNanos = TimeUnit.MILLISECONDS.toNanos(conf.getJournalMaxGroupWaitMSec());
         this.bufferedWritesThreshold = conf.getJournalBufferedWritesThreshold();
         this.bufferedEntriesThreshold = conf.getJournalBufferedEntriesThreshold();
@@ -682,11 +730,15 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         this.journalPageCacheFlushIntervalMSec = conf.getJournalPageCacheFlushIntervalMSec();
         if (conf.getNumJournalCallbackThreads() > 0) {
             this.cbThreadPool = Executors.newFixedThreadPool(conf.getNumJournalCallbackThreads(),
-                                                         new DefaultThreadFactory("bookie-journal-callback"));
+                                                        new CbThreadFactory());
+            this.callbackTime = journalStatsLogger.getThreadScopedCounter("callback-thread-time");
         } else {
             this.cbThreadPool = MoreExecutors.newDirectExecutorService();
+            this.callbackTime = journalStatsLogger.getThreadScopedCounter("callback-time");
         }
 
+        this.journalTime = journalStatsLogger.getThreadScopedCounter("journal-thread-time");
+
         // Unless there is a cap on the max wait (which requires group force writes)
         // we cannot skip flushing for queue empty
         this.flushWhenQueueEmpty = maxGroupWaitInNanos <= 0 || conf.getJournalFlushWhenQueueEmpty();
@@ -703,8 +755,16 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
             LOG.debug("Last Log Mark : {}", lastLogMark.getCurMark());
         }
 
+        try {
+            this.fileChannelProvider = FileChannelProvider.newProvider(conf.getJournalChannelProvider());
+        } catch (IOException e) {
+            LOG.error("Failed to initiate file channel provider: {}", conf.getJournalChannelProvider());
+            throw new RuntimeException(e);
+        }
+
         // Expose Stats
-        this.journalStats = new JournalStats(statsLogger);
+        this.journalStats = new JournalStats(journalStatsLogger, journalMaxMemory,
+                () -> memoryLimitController.currentUsage());
     }
 
     JournalStats getJournalStats() {
@@ -777,21 +837,21 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
     /**
      * Scan the journal.
      *
-     * @param journalId         Journal Log Id
-     * @param journalPos        Offset to start scanning
-     * @param scanner           Scanner to handle entries
-     * @param skipInvalidRecord when invalid record,should we skip it or not
+     * @param journalId Journal Log Id
+     * @param journalPos Offset to start scanning
+     * @param scanner Scanner to handle entries
      * @return scanOffset - represents the byte till which journal was read
      * @throws IOException
      */
-    public long scanJournal(long journalId, long journalPos, JournalScanner scanner, boolean skipInvalidRecord)
+    public long scanJournal(long journalId, long journalPos, JournalScanner scanner)
         throws IOException {
         JournalChannel recLog;
         if (journalPos <= 0) {
-            recLog = new JournalChannel(journalDirectory, journalId, journalPreAllocSize, journalWriteBufferSize);
+            recLog = new JournalChannel(journalDirectory, journalId, journalPreAllocSize, journalWriteBufferSize,
+                conf, fileChannelProvider);
         } else {
             recLog = new JournalChannel(journalDirectory, journalId, journalPreAllocSize, journalWriteBufferSize,
-                    journalPos);
+                    journalPos, conf, fileChannelProvider);
         }
         int journalVersion = recLog.getFormatVersion();
         try {
@@ -847,13 +907,6 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                 }
             }
             return recLog.fc.position();
-        } catch (IOException e) {
-            if (skipInvalidRecord) {
-                LOG.warn("Failed to parse journal file, and skipInvalidRecord is true, skip this journal file reply");
-            } else {
-                throw e;
-            }
-            return recLog.fc.position();
         } finally {
             recLog.close();
         }
@@ -889,15 +942,17 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         queue.put(QueueEntry.create(
                 entry, ackBeforeSync,  ledgerId, entryId, cb, ctx, MathUtils.nowInNano(),
                 journalStats.getJournalAddEntryStats(),
-                journalStats.getJournalCbQueueSize()));
+                journalStats.getJournalCbQueueSize(),
+                callbackTime));
     }
 
     void forceLedger(long ledgerId, WriteCallback cb, Object ctx) {
         queue.add(QueueEntry.create(
                 null, false /* ackBeforeSync */, ledgerId,
-                Bookie.METAENTRY_ID_FORCE_LEDGER, cb, ctx, MathUtils.nowInNano(),
+                BookieImpl.METAENTRY_ID_FORCE_LEDGER, cb, ctx, MathUtils.nowInNano(),
                 journalStats.getJournalForceLedgerStats(),
-                journalStats.getJournalCbQueueSize()));
+                journalStats.getJournalCbQueueSize(),
+                callbackTime));
         // Increment afterwards because the add operation could fail.
         journalStats.getJournalQueueSize().inc();
         journalStats.getJournalCbQueueSize().inc();
@@ -930,6 +985,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
     @Override
     public void run() {
         LOG.info("Starting journal on {}", journalDirectory);
+        ThreadRegistry.register(journalThreadName, 0);
 
         if (conf.isBusyWaitEnabled()) {
             try {
@@ -963,6 +1019,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
             long dequeueStartTime = 0L;
             long lastFlushTimeMs = System.currentTimeMillis();
 
+            long busyStartTime = System.nanoTime();
             QueueEntry qe = null;
             while (true) {
                 // new journal file to write
@@ -973,7 +1030,8 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                     journalCreationWatcher.reset().start();
                     logFile = new JournalChannel(journalDirectory, logId, journalPreAllocSize, journalWriteBufferSize,
                                         journalAlignmentSize, removePagesFromCache,
-                                        journalFormatVersionToWrite, getBufferedChannelBuilder());
+                                        journalFormatVersionToWrite, getBufferedChannelBuilder(),
+                                        conf, fileChannelProvider);
 
                     journalStats.getJournalCreationStats().registerSuccessfulEvent(
                             journalCreationWatcher.stop().elapsed(TimeUnit.NANOSECONDS), TimeUnit.NANOSECONDS);
@@ -990,8 +1048,10 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                     }
 
                     if (numEntriesToFlush == 0) {
+                        journalTime.add(MathUtils.elapsedNanos(busyStartTime));
                         qe = queue.take();
                         dequeueStartTime = MathUtils.nowInNano();
+                        busyStartTime = dequeueStartTime;
                         journalStats.getJournalQueueSize().dec();
                         journalStats.getJournalQueueStats()
                             .registerSuccessfulEvent(MathUtils.elapsedNanos(qe.enqueueTime), TimeUnit.NANOSECONDS);
@@ -1122,7 +1182,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                 if (qe == null) { // no more queue entry
                     continue;
                 }
-                if ((qe.entryId == Bookie.METAENTRY_ID_LEDGER_EXPLICITLAC)
+                if ((qe.entryId == BookieImpl.METAENTRY_ID_LEDGER_EXPLICITLAC)
                         && (journalFormatVersionToWrite < JournalChannel.V6)) {
                     /*
                      * this means we are using new code which supports
@@ -1133,7 +1193,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                      */
                     memoryLimitController.releaseMemory(qe.entry.readableBytes());
                     qe.entry.release();
-                } else if (qe.entryId != Bookie.METAENTRY_ID_FORCE_LEDGER) {
+                } else if (qe.entryId != BookieImpl.METAENTRY_ID_FORCE_LEDGER) {
                     int entrySize = qe.entry.readableBytes();
                     journalStats.getJournalWriteBytes().add(entrySize);
 
@@ -1184,6 +1244,10 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                 return;
             }
             LOG.info("Shutting down Journal");
+            if (fileChannelProvider != null) {
+                fileChannelProvider.close();
+            }
+
             forceWriteThread.shutdown();
             cbThreadPool.shutdown();
             if (!cbThreadPool.awaitTermination(5, TimeUnit.SECONDS)) {
@@ -1195,7 +1259,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
             this.interrupt();
             this.join();
             LOG.info("Finished Shutting down Journal thread");
-        } catch (InterruptedException ie) {
+        } catch (IOException | InterruptedException ie) {
             Thread.currentThread().interrupt();
             LOG.warn("Interrupted during shutting down journal : ", ie);
         }
