@@ -34,7 +34,6 @@ import org.apache.bookkeeper.bookie.storage.ldb.KeyValueStorage.Batch;
 import org.apache.bookkeeper.bookie.storage.ldb.KeyValueStorageFactory.DbConfigType;
 import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.stats.StatsLogger;
-import org.apache.bookkeeper.util.MathUtils;
 import org.apache.bookkeeper.util.collections.ConcurrentLongHashSet;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -48,15 +47,13 @@ import org.slf4j.LoggerFactory;
 public class EntryLocationIndex implements Closeable {
 
     private final KeyValueStorage locationsDb;
-    private final ConcurrentLongHashSet deletedLedgers = new ConcurrentLongHashSet();
-    private final int deleteEntriesBatchSize;
+    private final ConcurrentLongHashSet deletedLedgers = ConcurrentLongHashSet.newBuilder().build();
 
     private final EntryLocationIndexStats stats;
 
     public EntryLocationIndex(ServerConfiguration conf, KeyValueStorageFactory storageFactory, String basePath,
             StatsLogger stats) throws IOException {
-        locationsDb = storageFactory.newKeyValueStorage(basePath, "locations", DbConfigType.Huge, conf);
-        deleteEntriesBatchSize = conf.getRocksDBDeleteEntriesBatchSize();
+        locationsDb = storageFactory.newKeyValueStorage(basePath, "locations", DbConfigType.EntryLocation, conf);
 
         this.stats = new EntryLocationIndexStats(
             stats,
@@ -78,8 +75,6 @@ public class EntryLocationIndex implements Closeable {
         LongPairWrapper key = LongPairWrapper.get(ledgerId, entryId);
         LongWrapper value = LongWrapper.get();
 
-        long startTimeNanos = MathUtils.nowInNano();
-        boolean operationSuccess = false;
         try {
             if (locationsDb.get(key.array, value.array) < 0) {
                 if (log.isDebugEnabled()) {
@@ -87,27 +82,27 @@ public class EntryLocationIndex implements Closeable {
                 }
                 return 0;
             }
-            operationSuccess = true;
+
             return value.getValue();
         } finally {
             key.recycle();
             value.recycle();
-            if (operationSuccess) {
-                stats.getLookupEntryLocationStats()
-                        .registerSuccessfulEvent(MathUtils.elapsedNanos(startTimeNanos), TimeUnit.NANOSECONDS);
-            } else {
-                stats.getLookupEntryLocationStats()
-                        .registerFailedEvent(MathUtils.elapsedNanos(startTimeNanos), TimeUnit.NANOSECONDS);
-            }
         }
     }
 
     public long getLastEntryInLedger(long ledgerId) throws IOException {
         if (deletedLedgers.contains(ledgerId)) {
             // Ledger already deleted
-            return -1;
+            if (log.isDebugEnabled()) {
+                log.debug("Ledger {} already deleted in db", ledgerId);
+            }
+            /**
+             * when Ledger already deleted,
+             * throw Bookie.NoEntryException same like  the method
+             * {@link EntryLocationIndex.getLastEntryInLedgerInternal} solving ledgerId is not found.
+             * */
+            throw new Bookie.NoEntryException(ledgerId, -1);
         }
-
         return getLastEntryInLedgerInternal(ledgerId);
     }
 
@@ -187,17 +182,19 @@ public class EntryLocationIndex implements Closeable {
         deletedLedgers.add(ledgerId);
     }
 
+    private static final int DELETE_ENTRIES_BATCH_SIZE = 100000;
+
     public void removeOffsetFromDeletedLedgers() throws IOException {
+        LongPairWrapper firstKeyWrapper = LongPairWrapper.get(-1, -1);
+        LongPairWrapper lastKeyWrapper = LongPairWrapper.get(-1, -1);
+        LongPairWrapper keyToDelete = LongPairWrapper.get(-1, -1);
+
         Set<Long> ledgersToDelete = deletedLedgers.items();
 
         if (ledgersToDelete.isEmpty()) {
             return;
         }
 
-        LongPairWrapper firstKeyWrapper = LongPairWrapper.get(-1, -1);
-        LongPairWrapper lastKeyWrapper = LongPairWrapper.get(-1, -1);
-        LongPairWrapper keyToDelete = LongPairWrapper.get(-1, -1);
-
         log.info("Deleting indexes for ledgers: {}", ledgersToDelete);
         long startTime = System.nanoTime();
         long deletedEntries = 0;
@@ -210,6 +207,7 @@ public class EntryLocationIndex implements Closeable {
                 if (log.isDebugEnabled()) {
                     log.debug("Deleting indexes from ledger {}", ledgerId);
                 }
+
                 firstKeyWrapper.set(ledgerId, 0);
                 lastKeyWrapper.set(ledgerId, Long.MAX_VALUE);
 
@@ -234,7 +232,7 @@ public class EntryLocationIndex implements Closeable {
                 }
                 if (log.isDebugEnabled()) {
                     log.debug("Deleting index for ledger {} entries ({} -> {})",
-                        ledgerId, firstEntryId, lastEntryId);
+                            ledgerId, firstEntryId, lastEntryId);
                 }
 
                 // Iterate over all the keys and remove each of them
@@ -245,10 +243,9 @@ public class EntryLocationIndex implements Closeable {
                     }
                     batch.remove(keyToDelete.array);
                     ++deletedEntriesInBatch;
-                    ++deletedEntries;
                 }
 
-                if (deletedEntriesInBatch > deleteEntriesBatchSize) {
+                if (deletedEntriesInBatch > DELETE_ENTRIES_BATCH_SIZE) {
                     batch.flush();
                     batch.clear();
                     deletedEntriesInBatch = 0;
@@ -267,7 +264,7 @@ public class EntryLocationIndex implements Closeable {
         }
 
         log.info("Deleted indexes for {} entries from {} ledgers in {} seconds", deletedEntries, ledgersToDelete.size(),
-            TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - startTime) / 1000.0);
+                TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - startTime) / 1000.0);
 
         // Removed from pending set
         for (long ledgerId : ledgersToDelete) {
