@@ -22,12 +22,10 @@ package org.apache.bookkeeper.client;
 import io.netty.buffer.ByteBuf;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
-import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.atomic.AtomicBoolean;
@@ -42,6 +40,7 @@ import org.apache.bookkeeper.proto.BookkeeperInternalCallbacks.ReadEntryCallback
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+
 /**
  * A utility class to check the complete ledger and finds the UnderReplicated fragments if any.
  *
@@ -239,13 +238,7 @@ public class LedgerChecker {
             if (lastStored != LedgerHandle.INVALID_ENTRY_ID) {
                 throw new InvalidFragmentException();
             }
-
-            if (bookieWatcher.isBookieUnavailable(fragment.getAddress(bookieIndex))) {
-                // fragment is on this bookie, but already know it's unavailable, so skip the call
-                cb.operationComplete(BKException.Code.BookieHandleNotAvailableException, fragment);
-            } else {
-                cb.operationComplete(BKException.Code.OK, fragment);
-            }
+            cb.operationComplete(BKException.Code.OK, fragment);
         } else if (bookieWatcher.isBookieUnavailable(fragment.getAddress(bookieIndex))) {
             // fragment is on this bookie, but already know it's unavailable, so skip the call
             cb.operationComplete(BKException.Code.BookieHandleNotAvailableException, fragment);
@@ -345,7 +338,7 @@ public class LedgerChecker {
 
         FullLedgerCallback(long numFragments,
                 GenericCallback<Set<LedgerFragment>> cb) {
-            badFragments = new LinkedHashSet<>();
+            badFragments = new HashSet<LedgerFragment>();
             this.numFragments = new AtomicLong(numFragments);
             this.cb = cb;
         }
@@ -377,7 +370,7 @@ public class LedgerChecker {
                             final GenericCallback<Set<LedgerFragment>> cb,
                             long percentageOfLedgerFragmentToBeVerified) {
         // build a set of all fragment replicas
-        final Set<LedgerFragment> fragments = new LinkedHashSet<>();
+        final Set<LedgerFragment> fragments = new HashSet<LedgerFragment>();
 
         Long curEntryId = null;
         List<BookieId> curEnsemble = null;
@@ -427,11 +420,6 @@ public class LedgerChecker {
             if (curEntryId == lastEntry) {
                 final long entryToRead = curEntryId;
 
-                final CompletableFuture<Void> future = new CompletableFuture<>();
-                future.whenCompleteAsync((re, ex) -> {
-                    checkFragments(fragments, cb, percentageOfLedgerFragmentToBeVerified);
-                });
-
                 final EntryExistsCallback eecb = new EntryExistsCallback(lh.getLedgerMetadata().getWriteQuorumSize(),
                                               new GenericCallback<Boolean>() {
                                                   @Override
@@ -439,7 +427,8 @@ public class LedgerChecker {
                                                       if (result) {
                                                           fragments.add(lastLedgerFragment);
                                                       }
-                                                      future.complete(null);
+                                                      checkFragments(fragments, cb,
+                                                          percentageOfLedgerFragmentToBeVerified);
                                                   }
                                               });
 
