@@ -19,7 +19,6 @@ package org.apache.bookkeeper.conf;
 
 import static org.apache.bookkeeper.conf.ClientConfiguration.CLIENT_AUTH_PROVIDER_FACTORY_CLASS;
 
-import io.netty.buffer.PooledByteBufAllocator;
 import java.net.URL;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -81,6 +80,7 @@ public abstract class AbstractConfiguration<T extends AbstractConfiguration>
     // Ledger Manager
     protected static final String LEDGER_MANAGER_TYPE = "ledgerManagerType";
     protected static final String LEDGER_MANAGER_FACTORY_CLASS = "ledgerManagerFactoryClass";
+    protected static final String LEDGER_METADATA_FORMAT_VERSION = "ledgerMetadataVersion";
     protected static final String ALLOW_SHADED_LEDGER_MANAGER_FACTORY_CLASS = "allowShadedLedgerManagerFactoryClass";
     protected static final String SHADED_LEDGER_MANAGER_FACTORY_CLASS_PREFIX = "shadedLedgerManagerFactoryClassPrefix";
     protected static final String METADATA_SERVICE_URI = "metadataServiceUri";
@@ -93,6 +93,7 @@ public abstract class AbstractConfiguration<T extends AbstractConfiguration>
     protected static final String STORE_SYSTEMTIME_AS_LEDGER_CREATION_TIME = "storeSystemTimeAsLedgerCreationTime";
 
     protected static final String ENABLE_BUSY_WAIT = "enableBusyWait";
+    protected static final String ENABLE_HEALTH_CHECK = "enableHealthCheck";
 
     // Metastore settings, only being used when LEDGER_MANAGER_FACTORY_CLASS is MSLedgerManagerFactory
     protected static final String METASTORE_IMPL_CLASS = "metastoreImplClass";
@@ -465,6 +466,25 @@ public abstract class AbstractConfiguration<T extends AbstractConfiguration>
         return getString(LEDGER_MANAGER_FACTORY_CLASS);
     }
 
+    /**
+     * Set Ledger metadata format version.
+     *
+     * @param metadataFormatVersion
+     *          Ledger metadata format version. pass -1 to use default version
+     */
+    public void setLedgerMetadataFormatVersion(int metadataFormatVersion) {
+        setProperty(LEDGER_METADATA_FORMAT_VERSION, metadataFormatVersion);
+    }
+
+    /**
+     * Get Ledger metadata format version.
+     *
+     * @return ledger metadata format version.
+     */
+    public int getLedgerMetadataFormatVersion() {
+        return getInt(LEDGER_METADATA_FORMAT_VERSION, -1);
+    }
+
     /**
      * Get layout string ("null" if unconfigured).
      *
@@ -1065,7 +1085,7 @@ public abstract class AbstractConfiguration<T extends AbstractConfiguration>
      * @return the configured pooling concurrency for the allocator.
      */
     public int getAllocatorPoolingConcurrency() {
-        return this.getInteger(ALLOCATOR_POOLING_CONCURRENCY, PooledByteBufAllocator.defaultNumDirectArena());
+        return this.getInteger(ALLOCATOR_POOLING_CONCURRENCY, 2 * Runtime.getRuntime().availableProcessors());
     }
 
     /**
