@@ -17,6 +17,7 @@
  */
 package org.apache.bookkeeper.net;
 
+import com.google.common.base.Strings;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -45,7 +46,6 @@ public class NetworkTopologyImpl implements NetworkTopology {
     public static final int DEFAULT_HOST_LEVEL = 2;
     public static final Logger LOG = LoggerFactory.getLogger(NetworkTopologyImpl.class);
     public static final String NODE_SEPARATOR = ",";
-    public static final String INVERSE = "~";
 
     /**
      * A marker for an InvalidTopology Exception.
@@ -120,9 +120,10 @@ public class NetworkTopologyImpl implements NetworkTopology {
          * @return true if this node is an ancestor of <i>n</i>
          */
         boolean isAncestor(Node n) {
-            return getPath(this).equals(NodeBase.PATH_SEPARATOR_STR)
+            return !Strings.isNullOrEmpty(n.getNetworkLocation())
+                && (getPath(this).equals(NodeBase.PATH_SEPARATOR_STR)
                     || (n.getNetworkLocation() + NodeBase.PATH_SEPARATOR_STR).startsWith(getPath(this)
-                            + NodeBase.PATH_SEPARATOR_STR);
+                            + NodeBase.PATH_SEPARATOR_STR));
         }
 
         /**
@@ -708,7 +709,7 @@ public class NetworkTopologyImpl implements NetworkTopology {
     public Node chooseRandom(String scope) {
         netlock.readLock().lock();
         try {
-            if (scope.startsWith(INVERSE)) {
+            if (scope.startsWith("~")) {
                 return chooseRandom(NodeBase.ROOT, scope.substring(1));
             } else {
                 return chooseRandom(scope, null);
@@ -774,7 +775,7 @@ public class NetworkTopologyImpl implements NetworkTopology {
     public Set<Node> getLeaves(String scope) {
         netlock.readLock().lock();
         try {
-            if (scope.startsWith(INVERSE)) {
+            if (scope.startsWith("~")) {
                 Set<Node> allNodes = doGetLeaves(NodeBase.ROOT);
                 String[] excludeScopes = scope.substring(1).split(NODE_SEPARATOR);
                 Set<Node> excludeNodes = new HashSet<Node>();
@@ -794,7 +795,7 @@ public class NetworkTopologyImpl implements NetworkTopology {
     @Override
     public int countNumOfAvailableNodes(String scope, Collection<Node> excludedNodes) {
         boolean isExcluded = false;
-        if (scope.startsWith(INVERSE)) {
+        if (scope.startsWith("~")) {
             isExcluded = true;
             scope = scope.substring(1);
         }
