@@ -17,23 +17,19 @@
  */
 package org.apache.bookkeeper.proto;
 
-import com.google.common.util.concurrent.FutureCallback;
-import com.google.common.util.concurrent.Futures;
-import com.google.common.util.concurrent.ListenableFuture;
-import com.google.common.util.concurrent.SettableFuture;
 import io.netty.buffer.ByteBuf;
 import io.netty.channel.Channel;
 import io.netty.util.Recycler;
 import io.netty.util.ReferenceCountUtil;
-
 import java.io.IOException;
+import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
-
 import org.apache.bookkeeper.bookie.Bookie;
 import org.apache.bookkeeper.bookie.BookieException;
+import org.apache.bookkeeper.common.concurrent.FutureEventListener;
 import org.apache.bookkeeper.proto.BookieProtocol.ReadRequest;
 import org.apache.bookkeeper.stats.OpStatsLogger;
 import org.apache.bookkeeper.util.MathUtils;
@@ -56,7 +52,6 @@ class ReadEntryProcessor extends PacketProcessorBase<ReadRequest> {
         rep.init(request, channel, requestProcessor);
         rep.fenceThreadPool = fenceThreadPool;
         rep.throttleReadResponses = throttleReadResponses;
-        requestProcessor.onReadRequestStart(channel);
         return rep;
     }
 
@@ -65,19 +60,11 @@ class ReadEntryProcessor extends PacketProcessorBase<ReadRequest> {
         if (LOG.isDebugEnabled()) {
             LOG.debug("Received new read request: {}", request);
         }
-        if (!channel.isOpen()) {
-            if (LOG.isDebugEnabled()) {
-                LOG.debug("Dropping read request for closed channel: {}", channel);
-            }
-            requestProcessor.onReadRequestFinish();
-            recycle();
-            return;
-        }
         int errorCode = BookieProtocol.EOK;
         long startTimeNanos = MathUtils.nowInNano();
         ByteBuf data = null;
         try {
-            SettableFuture<Boolean> fenceResult = null;
+            CompletableFuture<Boolean> fenceResult = null;
             if (request.isFencing()) {
                 LOG.warn("Ledger: {}  fenced by: {}", request.getLedgerId(), channel.remoteAddress());
 
@@ -112,6 +99,9 @@ class ReadEntryProcessor extends PacketProcessorBase<ReadRequest> {
                 LOG.debug("Error reading {}", request, e);
             }
             errorCode = BookieProtocol.EIO;
+        } catch (BookieException.DataUnknownException e) {
+            LOG.error("Ledger {} is in an unknown state", request.getLedgerId(), e);
+            errorCode = BookieProtocol.EUNKNOWNLEDGERSTATE;
         } catch (BookieException e) {
             LOG.error("Unauthorized access to ledger {}", request.getLedgerId(), e);
             errorCode = BookieProtocol.EUA;
@@ -142,7 +132,11 @@ class ReadEntryProcessor extends PacketProcessorBase<ReadRequest> {
             response = ResponseBuilder.buildErrorResponse(errorCode, request);
         }
 
-        sendReadReqResponse(errorCode, response, stats.getReadRequestStats(), throttleReadResponses);
+        if (throttleReadResponses) {
+            sendResponseAndWait(errorCode, response, stats.getReadRequestStats());
+        } else {
+            sendResponse(errorCode, response, stats.getReadRequestStats());
+        }
         recycle();
     }
 
@@ -151,11 +145,11 @@ class ReadEntryProcessor extends PacketProcessorBase<ReadRequest> {
         sendResponse(data, retCode, startTimeNanos);
     }
 
-    private void handleReadResultForFenceRead(ListenableFuture<Boolean> fenceResult,
+    private void handleReadResultForFenceRead(CompletableFuture<Boolean> fenceResult,
                                               ByteBuf data,
                                               long startTimeNanos) {
         if (null != fenceThreadPool) {
-            Futures.addCallback(fenceResult, new FutureCallback<Boolean>() {
+            fenceResult.whenCompleteAsync(new FutureEventListener<Boolean>() {
                 @Override
                 public void onSuccess(Boolean result) {
                     sendFenceResponse(result, data, startTimeNanos);
