@@ -33,7 +33,6 @@ import com.google.common.cache.CacheLoader;
 import com.google.common.cache.LoadingCache;
 import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import java.io.IOException;
-import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.List;
@@ -43,7 +42,6 @@ import java.util.Set;
 import java.util.SortedMap;
 import java.util.Timer;
 import java.util.TimerTask;
-import java.util.TreeMap;
 import java.util.concurrent.ConcurrentSkipListSet;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
@@ -71,6 +69,7 @@ import org.apache.bookkeeper.stats.NullStatsLogger;
 import org.apache.bookkeeper.stats.OpStatsLogger;
 import org.apache.bookkeeper.stats.StatsLogger;
 import org.apache.bookkeeper.stats.annotations.StatsDoc;
+import org.apache.zookeeper.KeeperException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -143,7 +142,7 @@ public class ReplicationWorker implements Runnable {
      *            - configurations
      */
     public ReplicationWorker(final ServerConfiguration conf)
-            throws CompatibilityException, UnavailableException,
+            throws CompatibilityException, KeeperException,
             InterruptedException, IOException {
         this(conf, NullStatsLogger.INSTANCE);
     }
@@ -160,7 +159,8 @@ public class ReplicationWorker implements Runnable {
      */
     public ReplicationWorker(final ServerConfiguration conf,
                              StatsLogger statsLogger)
-            throws CompatibilityException, UnavailableException,
+            throws CompatibilityException, KeeperException,
+
             InterruptedException, IOException {
         this(conf, Auditor.createBookKeeperClient(conf), true, statsLogger);
     }
@@ -169,7 +169,8 @@ public class ReplicationWorker implements Runnable {
                       BookKeeper bkc,
                       boolean ownBkc,
                       StatsLogger statsLogger)
-            throws CompatibilityException, InterruptedException, UnavailableException {
+            throws CompatibilityException, KeeperException,
+            InterruptedException, IOException {
         this.conf = conf;
         this.bkc = bkc;
         this.ownBkc = ownBkc;
@@ -229,9 +230,12 @@ public class ReplicationWorker implements Runnable {
         workerRunning = true;
         while (workerRunning) {
             try {
-                rereplicate();
+                if (!rereplicate()) {
+                    LOG.warn("failed while replicating fragments");
+                    waitBackOffTime(rwRereplicateBackoffMs);
+                }
             } catch (InterruptedException e) {
-                LOG.info("InterruptedException "
+                LOG.error("InterruptedException "
                         + "while replicating fragments", e);
                 shutdown();
                 Thread.currentThread().interrupt();
@@ -239,11 +243,6 @@ public class ReplicationWorker implements Runnable {
             } catch (BKException e) {
                 LOG.error("BKException while replicating fragments", e);
                 waitBackOffTime(rwRereplicateBackoffMs);
-            } catch (ReplicationException.NonRecoverableReplicationException nre) {
-                LOG.error("NonRecoverableReplicationException "
-                        + "while replicating fragments", nre);
-                shutdown();
-                return;
             } catch (UnavailableException e) {
                 LOG.error("UnavailableException "
                         + "while replicating fragments", e);
@@ -270,7 +269,7 @@ public class ReplicationWorker implements Runnable {
      * Replicates the under replicated fragments from failed bookie ledger to
      * targetBookie.
      */
-    private void rereplicate() throws InterruptedException, BKException,
+    private boolean rereplicate() throws InterruptedException, BKException,
             UnavailableException {
         long ledgerIdToReplicate = underreplicationManager
                 .getLedgerToRereplicate();
@@ -287,6 +286,7 @@ public class ReplicationWorker implements Runnable {
                 rereplicateOpStats.registerFailedEvent(latencyMillis, TimeUnit.MILLISECONDS);
             }
         }
+        return success;
     }
 
     private void logBKExceptionAndReleaseLedger(BKException e, long ledgerIdToReplicate)
@@ -375,6 +375,7 @@ public class ReplicationWorker implements Runnable {
             }
 
             boolean foundOpenFragments = false;
+            long numFragsReplicated = 0;
             for (LedgerFragment ledgerFragment : fragments) {
                 if (!ledgerFragment.isClosed()) {
                     foundOpenFragments = true;
@@ -387,6 +388,7 @@ public class ReplicationWorker implements Runnable {
                 }
                 try {
                     admin.replicateLedgerFragment(lh, ledgerFragment, onReadEntryFailureCallback);
+                    numFragsReplicated++;
                 } catch (BKException.BKBookieHandleNotAvailableException e) {
                     LOG.warn("BKBookieHandleNotAvailableException while replicating the fragment", e);
                 } catch (BKException.BKLedgerRecoveryException e) {
@@ -395,6 +397,11 @@ public class ReplicationWorker implements Runnable {
                     LOG.warn("BKNotEnoughBookiesException while replicating the fragment", e);
                 }
             }
+
+            if (numFragsReplicated > 0) {
+                numLedgersReplicated.inc();
+            }
+
             if (foundOpenFragments || isLastSegmentOpenAndMissingBookies(lh)) {
                 deferLedgerLockRelease = true;
                 deferLedgerLockRelease(ledgerIdToReplicate);
@@ -466,14 +473,6 @@ public class ReplicationWorker implements Runnable {
      *
      * <p>Missing bookies in closed ledgers are fine, as we know the last confirmed add, so
      * we can tell which entries are supposed to exist and rereplicate them if necessary.
-     *
-     * <p>Another corner case is that there are multiple ensembles in the ledger and the last
-     * segment/ensemble is open, but nothing has been written to some quorums in the ensemble.
-     * For the v2 protocol, this ledger's lastAddConfirm entry is the last segment/ensemble's `key - 2`,
-     * not `key - 2`, the explanation please refer to: https://github.com/apache/bookkeeper/pull/3917.
-     * If we treat the penultimate segment/ensemble as closed state, we will can't replicate
-     * the last entry in the segment. So in this case, we should also check if the penultimate
-     * segment/ensemble has missing bookies.
      */
     private boolean isLastSegmentOpenAndMissingBookies(LedgerHandle lh) throws BKException {
         LedgerMetadata md = admin.getLedgerMetadata(lh);
@@ -483,10 +482,6 @@ public class ReplicationWorker implements Runnable {
 
         SortedMap<Long, ? extends List<BookieId>> ensembles = admin.getLedgerMetadata(lh).getAllEnsembles();
         List<BookieId> finalEnsemble = ensembles.get(ensembles.lastKey());
-        if (ensembles.size() > 1 && lh.getLastAddConfirmed() < ensembles.lastKey() - 1) {
-            finalEnsemble = new ArrayList<>(finalEnsemble);
-            finalEnsemble.addAll((new TreeMap<>(ensembles)).floorEntry(ensembles.lastKey() - 1).getValue());
-        }
         Collection<BookieId> available = admin.getAvailableBookies();
         for (BookieId b : finalEnsemble) {
             if (!available.contains(b)) {
