@@ -50,6 +50,7 @@ import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.mockito.Mockito;
 import org.powermock.api.mockito.PowerMockito;
+import org.powermock.core.classloader.annotations.PowerMockIgnore;
 import org.powermock.core.classloader.annotations.PrepareForTest;
 import org.powermock.modules.junit4.PowerMockRunner;
 import org.slf4j.Logger;
@@ -59,7 +60,8 @@ import org.slf4j.LoggerFactory;
  * Test the bookie journal.
  */
 @RunWith(PowerMockRunner.class)
-@PrepareForTest(JournalChannel.class)
+@PrepareForTest({JournalChannel.class, FileChannelProvider.class})
+@PowerMockIgnore({"jdk.internal.loader.*", "javax.naming.*"})
 public class BookieJournalTest {
     private static final Logger LOG = LoggerFactory.getLogger(BookieJournalTest.class);
 
@@ -121,7 +123,7 @@ public class BookieJournalTest {
     private static ByteBuf generateFenceEntry(long ledgerId) {
         ByteBuf bb = Unpooled.buffer();
         bb.writeLong(ledgerId);
-        bb.writeLong(Bookie.METAENTRY_ID_FENCE_KEY);
+        bb.writeLong(BookieImpl.METAENTRY_ID_FENCE_KEY);
         return bb;
     }
 
@@ -131,7 +133,7 @@ public class BookieJournalTest {
     private static ByteBuf generateMetaEntry(long ledgerId, byte[] masterKey) {
         ByteBuf bb = Unpooled.buffer();
         bb.writeLong(ledgerId);
-        bb.writeLong(Bookie.METAENTRY_ID_LEDGER_KEY);
+        bb.writeLong(BookieImpl.METAENTRY_ID_LEDGER_KEY);
         bb.writeInt(masterKey.length);
         bb.writeBytes(masterKey);
         return bb;
@@ -297,52 +299,6 @@ public class BookieJournalTest {
         return jc;
     }
 
-    private JournalChannel writeV4JournalWithInvalidRecord(File journalDir,
-                                                           int numEntries, byte[] masterKey) throws Exception {
-        long logId = System.currentTimeMillis();
-        JournalChannel jc = new JournalChannel(journalDir, logId);
-
-        moveToPosition(jc, JournalChannel.VERSION_HEADER_SIZE);
-
-        BufferedChannel bc = jc.getBufferedChannel();
-
-        byte[] data = new byte[1024];
-        Arrays.fill(data, (byte) 'X');
-        long lastConfirmed = LedgerHandle.INVALID_ENTRY_ID;
-        for (int i = 0; i <= numEntries; i++) {
-            ByteBuf packet;
-            if (i == 0) {
-                packet = generateMetaEntry(1, masterKey);
-            } else {
-                packet = ClientUtil.generatePacket(1, i, lastConfirmed, i * data.length, data);
-            }
-            lastConfirmed = i;
-            ByteBuffer lenBuff = ByteBuffer.allocate(4);
-            if (i == numEntries - 1) {
-                //mock when flush data to file ,it writes an invalid entry to journal
-                lenBuff.putInt(-1);
-            } else {
-                lenBuff.putInt(packet.readableBytes());
-            }
-            lenBuff.flip();
-            bc.write(Unpooled.wrappedBuffer(lenBuff));
-            bc.write(packet);
-            packet.release();
-        }
-
-        // write fence key
-        ByteBuf packet = generateFenceEntry(1);
-        ByteBuf lenBuf = Unpooled.buffer();
-        lenBuf.writeInt(packet.readableBytes());
-        //mock
-        bc.write(lenBuf);
-        bc.write(packet);
-        bc.flushAndForceWrite(false);
-        updateJournalVersion(jc, JournalChannel.V4);
-
-        return jc;
-    }
-
     static JournalChannel writeV5Journal(File journalDir, int numEntries,
                                          byte[] masterKey) throws Exception {
         return writeV5Journal(journalDir, numEntries, masterKey, false);
@@ -401,13 +357,13 @@ public class BookieJournalTest {
     @Test
     public void testPreV2Journal() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
-        writePreV2Journal(Bookie.getCurrentDirectory(journalDir), 100);
-        writeIndexFileForLedger(Bookie.getCurrentDirectory(ledgerDir), 1, "testPasswd".getBytes());
+        writePreV2Journal(BookieImpl.getCurrentDirectory(journalDir), 100);
+        writeIndexFileForLedger(BookieImpl.getCurrentDirectory(ledgerDir), 1, "testPasswd".getBytes());
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
@@ -430,19 +386,19 @@ public class BookieJournalTest {
     @Test
     public void testV4Journal() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
-        writeV4Journal(Bookie.getCurrentDirectory(journalDir), 100, "testPasswd".getBytes());
+        writeV4Journal(BookieImpl.getCurrentDirectory(journalDir), 100, "testPasswd".getBytes());
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
             .setLedgerDirNames(new String[] { ledgerDir.getPath() })
             .setMetadataServiceUri(null);
 
-        Bookie b = createBookieAndReadJournal(conf);
+        BookieImpl b = createBookieAndReadJournal(conf);
 
         b.readEntry(1, 100);
         try {
@@ -459,12 +415,12 @@ public class BookieJournalTest {
     @Test
     public void testV5Journal() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
-        writeV5Journal(Bookie.getCurrentDirectory(journalDir), 2 * JournalChannel.SECTOR_SIZE,
+        writeV5Journal(BookieImpl.getCurrentDirectory(journalDir), 2 * JournalChannel.SECTOR_SIZE,
                 "testV5Journal".getBytes());
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
@@ -472,7 +428,7 @@ public class BookieJournalTest {
             .setLedgerDirNames(new String[] { ledgerDir.getPath() })
             .setMetadataServiceUri(null);
 
-        Bookie b = createBookieAndReadJournal(conf);
+        BookieImpl b = createBookieAndReadJournal(conf);
 
         for (int i = 1; i <= 2 * JournalChannel.SECTOR_SIZE; i++) {
             b.readEntry(1, i);
@@ -496,12 +452,12 @@ public class BookieJournalTest {
     @Test
     public void testAllJunkJournal() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
-        writeJunkJournal(Bookie.getCurrentDirectory(journalDir));
+        writeJunkJournal(BookieImpl.getCurrentDirectory(journalDir));
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
@@ -510,7 +466,7 @@ public class BookieJournalTest {
 
         Bookie b = null;
         try {
-            b = new Bookie(conf);
+            b = new TestBookieImpl(conf);
             fail("Shouldn't have been able to start without admin");
         } catch (Throwable t) {
             // correct behaviour
@@ -531,19 +487,19 @@ public class BookieJournalTest {
     @Test
     public void testEmptyJournal() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
-        writePreV2Journal(Bookie.getCurrentDirectory(journalDir), 0);
+        writePreV2Journal(BookieImpl.getCurrentDirectory(journalDir), 0);
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
             .setLedgerDirNames(new String[] { ledgerDir.getPath() })
             .setMetadataServiceUri(null);
 
-        Bookie b = new Bookie(conf);
+        Bookie b = new TestBookieImpl(conf);
     }
 
     /**
@@ -553,19 +509,19 @@ public class BookieJournalTest {
     @Test
     public void testHeaderOnlyJournal() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
-        writeV2Journal(Bookie.getCurrentDirectory(journalDir), 0);
+        writeV2Journal(BookieImpl.getCurrentDirectory(journalDir), 0);
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
             .setLedgerDirNames(new String[] { ledgerDir.getPath() })
             .setMetadataServiceUri(null);
 
-        Bookie b = new Bookie(conf);
+        Bookie b = new TestBookieImpl(conf);
     }
 
     /**
@@ -575,15 +531,14 @@ public class BookieJournalTest {
     @Test
     public void testJunkEndedJournal() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
-        JournalChannel jc = writeV2Journal(Bookie.getCurrentDirectory(journalDir), 0);
+        JournalChannel jc = writeV2Journal(BookieImpl.getCurrentDirectory(journalDir), 0);
         jc.getBufferedChannel().write(Unpooled.wrappedBuffer("JunkJunkJunk".getBytes()));
         jc.getBufferedChannel().flushAndForceWrite(false);
-        jc.close();
 
         writeIndexFileForLedger(ledgerDir, 1, "testPasswd".getBytes());
 
@@ -594,7 +549,7 @@ public class BookieJournalTest {
 
         Bookie b = null;
         try {
-            b = new Bookie(conf);
+            b = new TestBookieImpl(conf);
         } catch (Throwable t) {
             // correct behaviour
         }
@@ -611,20 +566,20 @@ public class BookieJournalTest {
     @Test
     public void testTruncatedInLenJournal() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
         JournalChannel jc = writeV2Journal(
-                Bookie.getCurrentDirectory(journalDir), 100);
+                BookieImpl.getCurrentDirectory(journalDir), 100);
         ByteBuffer zeros = ByteBuffer.allocate(2048);
 
         jc.fc.position(jc.getBufferedChannel().position() - 0x429);
         jc.fc.write(zeros);
         jc.fc.force(false);
 
-        writeIndexFileForLedger(Bookie.getCurrentDirectory(ledgerDir),
+        writeIndexFileForLedger(BookieImpl.getCurrentDirectory(ledgerDir),
                                 1, "testPasswd".getBytes());
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
@@ -654,20 +609,20 @@ public class BookieJournalTest {
     @Test
     public void testTruncatedInEntryJournal() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
         JournalChannel jc = writeV2Journal(
-                Bookie.getCurrentDirectory(journalDir), 100);
+                BookieImpl.getCurrentDirectory(journalDir), 100);
         ByteBuffer zeros = ByteBuffer.allocate(2048);
 
         jc.fc.position(jc.getBufferedChannel().position() - 0x300);
         jc.fc.write(zeros);
         jc.fc.force(false);
 
-        writeIndexFileForLedger(Bookie.getCurrentDirectory(ledgerDir),
+        writeIndexFileForLedger(BookieImpl.getCurrentDirectory(ledgerDir),
                                 1, "testPasswd".getBytes());
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
@@ -701,8 +656,8 @@ public class BookieJournalTest {
         }
     }
 
-    private Bookie createBookieAndReadJournal(ServerConfiguration conf) throws Exception {
-        Bookie b = new Bookie(conf);
+    private BookieImpl createBookieAndReadJournal(ServerConfiguration conf) throws Exception {
+        BookieImpl b = new TestBookieImpl(conf);
         for (Journal journal : b.journals) {
             LastLogMark lastLogMark = journal.getLastLogMark().markLog();
             b.readJournal();
@@ -718,17 +673,17 @@ public class BookieJournalTest {
     @Test
     public void testSortedLedgerStorageReplayWithSmallMaxArenaSize() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
         JournalChannel jc = writeV2Journal(
-                Bookie.getCurrentDirectory(journalDir), 100);
+                BookieImpl.getCurrentDirectory(journalDir), 100);
 
         jc.fc.force(false);
 
-        writeIndexFileForLedger(Bookie.getCurrentDirectory(ledgerDir),
+        writeIndexFileForLedger(BookieImpl.getCurrentDirectory(ledgerDir),
                 1, "testPasswd".getBytes());
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
@@ -737,7 +692,7 @@ public class BookieJournalTest {
         conf.setJournalDirName(journalDir.getPath())
                 .setLedgerDirNames(new String[] { ledgerDir.getPath() });
 
-        Bookie b = new Bookie(conf);
+        BookieImpl b = new TestBookieImpl(conf);
         b.readJournal();
         b.ledgerStorage.flush();
         b.readEntry(1, 80);
@@ -766,13 +721,13 @@ public class BookieJournalTest {
     private void testPartialFileInfoPreV3Journal(boolean truncateMasterKey)
         throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
-        writePreV2Journal(Bookie.getCurrentDirectory(journalDir), 100);
-        writePartialIndexFileForLedger(Bookie.getCurrentDirectory(ledgerDir),
+        writePreV2Journal(BookieImpl.getCurrentDirectory(journalDir), 100);
+        writePartialIndexFileForLedger(BookieImpl.getCurrentDirectory(ledgerDir),
                                        1, "testPasswd".getBytes(), truncateMasterKey);
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
@@ -782,13 +737,13 @@ public class BookieJournalTest {
 
         if (truncateMasterKey) {
             try {
-                Bookie b = new Bookie(conf);
+                BookieImpl b = new TestBookieImpl(conf);
                 b.readJournal();
                 fail("Should not reach here!");
             } catch (IOException ie) {
             }
         } else {
-            Bookie b = new Bookie(conf);
+            BookieImpl b = new TestBookieImpl(conf);
             b.readJournal();
             b.readEntry(1, 100);
             try {
@@ -822,15 +777,15 @@ public class BookieJournalTest {
     private void testPartialFileInfoPostV3Journal(boolean truncateMasterKey)
         throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
         byte[] masterKey = "testPasswd".getBytes();
 
-        writeV3Journal(Bookie.getCurrentDirectory(journalDir), 100, masterKey);
-        writePartialIndexFileForLedger(Bookie.getCurrentDirectory(ledgerDir), 1, masterKey,
+        writeV3Journal(BookieImpl.getCurrentDirectory(journalDir), 100, masterKey);
+        writePartialIndexFileForLedger(BookieImpl.getCurrentDirectory(ledgerDir), 1, masterKey,
                                        truncateMasterKey);
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
@@ -838,7 +793,7 @@ public class BookieJournalTest {
             .setLedgerDirNames(new String[] { ledgerDir.getPath() })
             .setMetadataServiceUri(null);
 
-        Bookie b = new Bookie(conf);
+        BookieImpl b = new TestBookieImpl(conf);
         b.readJournal();
         b.readEntry(1, 100);
         try {
@@ -855,12 +810,12 @@ public class BookieJournalTest {
     @Test
     public void testJournalScanIOException() throws Exception {
         File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
 
-        writeV4Journal(Bookie.getCurrentDirectory(journalDir), 100, "testPasswd".getBytes());
+        writeV4Journal(BookieImpl.getCurrentDirectory(journalDir), 100, "testPasswd".getBytes());
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
@@ -868,15 +823,19 @@ public class BookieJournalTest {
                 .setMetadataServiceUri(null);
 
         Journal.JournalScanner journalScanner = new DummyJournalScan();
+        BookieFileChannel bookieFileChannel = PowerMockito.mock(BookieFileChannel.class);
         FileChannel fileChannel = PowerMockito.mock(FileChannel.class);
+        FileChannelProvider fileChannelProvider = PowerMockito.mock(FileChannelProvider.class);
 
         PowerMockito.when(fileChannel.position(Mockito.anyLong()))
                 .thenThrow(new IOException());
 
-        PowerMockito.mockStatic(JournalChannel.class);
-        PowerMockito.when(JournalChannel.openFileChannel(Mockito.any(RandomAccessFile.class))).thenReturn(fileChannel);
+        PowerMockito.mockStatic(FileChannelProvider.class);
+        PowerMockito.when(FileChannelProvider.newProvider(Mockito.any())).thenReturn(fileChannelProvider);
+        Mockito.when(fileChannelProvider.open(Mockito.any(), Mockito.any())).thenReturn(bookieFileChannel);
+        Mockito.when(bookieFileChannel.getFileChannel()).thenReturn(fileChannel);
 
-        Bookie b = new Bookie(conf);
+        BookieImpl b = new TestBookieImpl(conf);
 
         for (Journal journal : b.journals) {
             List<Long> journalIds = journal.listJournalIds(journal.getJournalDirectory(), null);
@@ -884,7 +843,7 @@ public class BookieJournalTest {
             assertEquals(journalIds.size(), 1);
 
             try {
-                journal.scanJournal(journalIds.get(0), Long.MAX_VALUE, journalScanner, false);
+                journal.scanJournal(journalIds.get(0), Long.MAX_VALUE, journalScanner);
                 fail("Should not have been able to scan the journal");
             } catch (Exception e) {
                 // Expected
@@ -894,74 +853,7 @@ public class BookieJournalTest {
         b.shutdown();
     }
 
-    /**
-     * Test for invalid record data during read of Journal.
-     */
-    @Test
-    public void testJournalScanInvalidRecordWithSkipFlag() throws Exception {
-        File journalDir = createTempDir("bookie", "journal");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
-
-        File ledgerDir = createTempDir("bookie", "ledger");
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
-
-        try {
-            writeV4JournalWithInvalidRecord(Bookie.getCurrentDirectory(journalDir),
-                100, "testPasswd".getBytes());
-        } catch (Exception e) {
-            fail();
-        }
-
-
-        ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
-        // Disabled skip broken journal files by default
-        conf.setJournalDirName(journalDir.getPath())
-                .setLedgerDirNames(new String[] { ledgerDir.getPath() })
-                .setMetadataServiceUri(null)
-                .setSkipReplayJournalInvalidRecord(true);
-
-        Journal.JournalScanner journalScanner = new DummyJournalScan();
-
-        Bookie b = new Bookie(conf);
-
-        for (Journal journal : b.journals) {
-            List<Long> journalIds = Journal.listJournalIds(journal.getJournalDirectory(), null);
-            assertEquals(journalIds.size(), 1);
-            try {
-                journal.scanJournal(journalIds.get(0), 0, journalScanner, conf.isSkipReplayJournalInvalidRecord());
-            } catch (Exception e) {
-                fail("Should pass the journal scanning because we enabled skip flag by default.");
-            }
-        }
-
-        b.shutdown();
-
-        // Disabled skip broken journal files by default
-        conf = TestBKConfiguration.newServerConfiguration();
-        conf.setJournalDirName(journalDir.getPath())
-                .setLedgerDirNames(new String[] { ledgerDir.getPath() })
-                .setMetadataServiceUri(null);
-
-        journalScanner = new DummyJournalScan();
-
-        b = new Bookie(conf);
-
-        for (Journal journal : b.journals) {
-            List<Long> journalIds = Journal.listJournalIds(journal.getJournalDirectory(), null);
-            assertEquals(journalIds.size(), 1);
-            try {
-                journal.scanJournal(journalIds.get(0), 0, journalScanner, conf.isSkipReplayJournalInvalidRecord());
-                fail("Should fail the journal scanning because of disabled skip flag");
-            } catch (Exception e) {
-                // expected.
-            }
-        }
-
-        b.shutdown();
-    }
-
-
-    static class DummyJournalScan implements Journal.JournalScanner {
+    private class DummyJournalScan implements Journal.JournalScanner {
 
         @Override
         public void process(int journalVersion, long offset, ByteBuffer entry) throws IOException {
