@@ -21,18 +21,19 @@
 
 package org.apache.bookkeeper.bookie;
 
-import static org.apache.bookkeeper.bookie.UpgradeTest.newV1JournalDirectory;
-import static org.apache.bookkeeper.bookie.UpgradeTest.newV1LedgerDirectory;
-import static org.apache.bookkeeper.bookie.UpgradeTest.newV2JournalDirectory;
-import static org.apache.bookkeeper.bookie.UpgradeTest.newV2LedgerDirectory;
+import static org.apache.bookkeeper.bookie.UpgradeTest.initV1JournalDirectory;
+import static org.apache.bookkeeper.bookie.UpgradeTest.initV1LedgerDirectory;
+import static org.apache.bookkeeper.bookie.UpgradeTest.initV2JournalDirectory;
+import static org.apache.bookkeeper.bookie.UpgradeTest.initV2LedgerDirectory;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
-
 import com.google.common.collect.Sets;
 import java.io.File;
 import java.io.IOException;
 import java.net.URI;
+import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
@@ -46,7 +47,6 @@ import org.apache.bookkeeper.meta.MetadataDrivers;
 import org.apache.bookkeeper.stats.NullStatsLogger;
 import org.apache.bookkeeper.test.BookKeeperClusterTestCase;
 import org.apache.bookkeeper.util.BookKeeperConstants;
-import org.apache.bookkeeper.util.IOUtils;
 import org.apache.bookkeeper.util.PortManager;
 import org.apache.bookkeeper.versioning.LongVersion;
 import org.apache.bookkeeper.versioning.Version;
@@ -54,26 +54,30 @@ import org.apache.bookkeeper.versioning.Versioned;
 import org.apache.commons.io.FileUtils;
 import org.junit.Test;
 
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
 /**
  * Test cookies.
  */
 public class CookieTest extends BookKeeperClusterTestCase {
+    private static final Logger log = LoggerFactory.getLogger(CookieTest.class);
+
     final int bookiePort = PortManager.nextFreePort();
 
     public CookieTest() {
         super(0);
     }
 
-    private String newDirectory() throws IOException {
+    private String newDirectory() throws Exception {
         return newDirectory(true);
     }
 
-    private String newDirectory(boolean createCurDir) throws IOException {
-        File d = IOUtils.createTempDir("cookie", "tmpdir");
+    private String newDirectory(boolean createCurDir) throws Exception {
+        File d = tmpDirs.createNew("cookie", "tmpdir");
         if (createCurDir) {
             new File(d, "current").mkdirs();
         }
-        tmpDirs.add(d);
         return d.getPath();
     }
 
@@ -86,33 +90,54 @@ public class CookieTest extends BookKeeperClusterTestCase {
         baseConf.setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         this.metadataBookieDriver = MetadataDrivers.getBookieDriver(
             URI.create(baseConf.getMetadataServiceUri()));
-        this.metadataBookieDriver.initialize(baseConf, () -> {}, NullStatsLogger.INSTANCE);
-        this.rm = metadataBookieDriver.getRegistrationManager();
+        this.metadataBookieDriver.initialize(baseConf, NullStatsLogger.INSTANCE);
+        this.rm = metadataBookieDriver.createRegistrationManager();
     }
 
     @Override
     public void tearDown() throws Exception {
         super.tearDown();
+        if (rm != null) {
+            rm.close();
+        }
         if (metadataBookieDriver != null) {
             metadataBookieDriver.close();
         }
     }
 
+    private static List<File> currentDirectoryList(File[] dirs) {
+        return Arrays.asList(BookieImpl.getCurrentDirectories(dirs));
+    }
+
+    private void validateConfig(ServerConfiguration conf) throws Exception {
+        List<File> dirs = new ArrayList<>();
+        for (File f : conf.getJournalDirs()) {
+            File cur = BookieImpl.getCurrentDirectory(f);
+            dirs.add(cur);
+            BookieImpl.checkDirectoryStructure(cur);
+        }
+        for (File f : conf.getLedgerDirs()) {
+            File cur = BookieImpl.getCurrentDirectory(f);
+            dirs.add(cur);
+            BookieImpl.checkDirectoryStructure(cur);
+        }
+        LegacyCookieValidation cookieValidation = new LegacyCookieValidation(conf, rm);
+        cookieValidation.checkCookies(dirs);
+
+    }
+
     /**
      * Test starting bookie with clean state.
      */
     @Test
     public void testCleanStart() throws Exception {
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
-        conf.setJournalDirName(newDirectory(false))
-            .setLedgerDirNames(new String[] { newDirectory(false) })
+        conf.setJournalDirName(newDirectory(true))
+            .setLedgerDirNames(new String[] { newDirectory(true) })
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
-        try {
-            Bookie b = new Bookie(conf);
-        } catch (Exception e) {
-            fail("Should not reach here.");
-        }
+
+        validateConfig(conf);
     }
 
     /**
@@ -143,7 +168,8 @@ public class CookieTest extends BookKeeperClusterTestCase {
         c2.writeToDirectory(new File(ledgerDir, "current"));
 
         try {
-            Bookie b = new Bookie(conf2);
+            validateConfig(conf2);
+
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behaviour
@@ -166,13 +192,11 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
 
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
 
         conf.setLedgerDirNames(new String[] { ledgerDirs[0], ledgerDirs[1] });
         try {
-            Bookie b2 = new Bookie(conf);
+            validateConfig(conf);
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behaviour
@@ -180,16 +204,14 @@ public class CookieTest extends BookKeeperClusterTestCase {
 
         conf.setJournalDirName(newDirectory()).setLedgerDirNames(ledgerDirs);
         try {
-            Bookie b2 = new Bookie(conf);
+            validateConfig(conf);
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behaviour
         }
 
         conf.setJournalDirName(journalDir);
-        b = new Bookie(conf);
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
     }
 
     /**
@@ -207,15 +229,13 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
 
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
 
         File cookieFile =
-            new File(Bookie.getCurrentDirectory(new File(journalDir)), BookKeeperConstants.VERSION_FILENAME);
+            new File(BookieImpl.getCurrentDirectory(new File(journalDir)), BookKeeperConstants.VERSION_FILENAME);
         assertTrue(cookieFile.delete());
         try {
-            new Bookie(conf);
+            validateConfig(conf);
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behaviour
@@ -237,15 +257,13 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
 
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
 
         File cookieFile =
-            new File(Bookie.getCurrentDirectory(new File(ledgerDirs[0])), BookKeeperConstants.VERSION_FILENAME);
+            new File(BookieImpl.getCurrentDirectory(new File(ledgerDirs[0])), BookKeeperConstants.VERSION_FILENAME);
         assertTrue(cookieFile.delete());
         try {
-            new Bookie(conf);
+            validateConfig(conf);
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behaviour
@@ -267,22 +285,18 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
 
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
 
         conf.setLedgerDirNames(new String[] { ledgerDir0, newDirectory() });
         try {
-            Bookie b2 = new Bookie(conf);
+            validateConfig(conf);
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behaviour
         }
 
         conf.setLedgerDirNames(new String[] { ledgerDir0 });
-        b = new Bookie(conf);
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
     }
 
     /**
@@ -302,10 +316,7 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setAllowStorageExpansion(true)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
 
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
-        b = null;
+        validateConfig(conf);
 
         // add a few additional ledger dirs
         String[] lPaths = new String[] {ledgerDir0, newDirectory(), newDirectory()};
@@ -318,12 +329,12 @@ public class CookieTest extends BookKeeperClusterTestCase {
         conf.setIndexDirName(iPaths);
 
         try {
-            b = new Bookie(conf);
+            validateConfig(conf);
         } catch (BookieException.InvalidCookieException ice) {
             fail("Should have been able to start the bookie");
         }
 
-        List<File> l = b.getLedgerDirsManager().getAllLedgerDirs();
+        List<File> l = currentDirectoryList(conf.getLedgerDirs());
         HashSet<String> bookieLedgerDirs = Sets.newHashSet();
         for (File f : l) {
             // Using the parent path because the bookie creates a 'current'
@@ -334,7 +345,7 @@ public class CookieTest extends BookKeeperClusterTestCase {
                    + bookieLedgerDirs,
                    configuredLedgerDirs.equals(bookieLedgerDirs));
 
-        l = b.getIndexDirsManager().getAllLedgerDirs();
+        l = currentDirectoryList(conf.getIndexDirs());
         HashSet<String> bookieIndexDirs = Sets.newHashSet();
         for (File f : l) {
             bookieIndexDirs.add(f.getParent());
@@ -343,14 +354,12 @@ public class CookieTest extends BookKeeperClusterTestCase {
                    + bookieIndexDirs,
                    configuredIndexDirs.equals(bookieIndexDirs));
 
-        b.shutdown();
-
         // Make sure that substituting an older ledger directory
         // is not allowed.
         String[] lPaths2 = new String[] { lPaths[0], lPaths[1], newDirectory() };
         conf.setLedgerDirNames(lPaths2);
         try {
-            b = new Bookie(conf);
+            validateConfig(conf);
             fail("Should not have been able to start the bookie");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behavior
@@ -361,7 +370,7 @@ public class CookieTest extends BookKeeperClusterTestCase {
         lPaths2 = new String[] { lPaths[0], lPaths[1] };
         conf.setLedgerDirNames(lPaths2);
         try {
-            b = new Bookie(conf);
+            validateConfig(conf);
             fail("Should not have been able to start the bookie");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behavior
@@ -385,22 +394,19 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setAllowStorageExpansion(true)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
 
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
-        b = null;
+        validateConfig(conf);
 
         // add an additional ledger dir
         String[] lPaths = new String[] {ledgerDir0, newDirectory()};
         conf.setLedgerDirNames(lPaths);
 
         // create a file to make the dir non-empty
-        File currentDir = Bookie.getCurrentDirectory(new File(lPaths[1]));
+        File currentDir = BookieImpl.getCurrentDirectory(new File(lPaths[1]));
         new File(currentDir, "foo").createNewFile();
         assertTrue(currentDir.list().length == 1);
 
         try {
-            b = new Bookie(conf);
+            validateConfig(conf);
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behavior
@@ -411,12 +417,12 @@ public class CookieTest extends BookKeeperClusterTestCase {
         conf.setIndexDirName(iPaths);
 
         // create a dir to make it non-empty
-        currentDir = Bookie.getCurrentDirectory(new File(iPaths[1]));
+        currentDir = BookieImpl.getCurrentDirectory(new File(iPaths[1]));
         new File(currentDir, "bar").mkdirs();
         assertTrue(currentDir.list().length == 1);
 
         try {
-            b = new Bookie(conf);
+            validateConfig(conf);
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behavior
@@ -437,13 +443,11 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
 
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
 
         FileUtils.deleteDirectory(new File(ledgerDir0));
         try {
-            Bookie b2 = new Bookie(conf);
+            validateConfig(conf);
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behaviour
@@ -461,13 +465,11 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setLedgerDirNames(new String[] { newDirectory() , newDirectory() })
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
 
         conf.setBookiePort(3182);
         try {
-            b = new Bookie(conf);
+            validateConfig(conf);
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behaviour
@@ -487,9 +489,7 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setLedgerDirNames(new String[] { newDirectory() , newDirectory() })
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
 
         conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(newDirectory())
@@ -497,7 +497,7 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         try {
-            b = new Bookie(conf);
+            validateConfig(conf);
             fail("Shouldn't have been able to start");
         } catch (BookieException.InvalidCookieException ice) {
             // correct behaviour
@@ -517,17 +517,17 @@ public class CookieTest extends BookKeeperClusterTestCase {
         BookKeeperAdmin.format(adminConf, false, true);
 
         ServerConfiguration bookieConf = TestBKConfiguration.newServerConfiguration();
-        bookieConf.setJournalDirName(newDirectory(false))
-            .setLedgerDirNames(new String[] { newDirectory(false) })
+        bookieConf.setJournalDirName(newDirectory(true))
+            .setLedgerDirNames(new String[] { newDirectory(true) })
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         // Bookie should start successfully for fresh env.
-        new Bookie(bookieConf);
+        validateConfig(bookieConf);
 
         // Format metadata one more time.
         BookKeeperAdmin.format(adminConf, false, true);
         try {
-            new Bookie(bookieConf);
+            validateConfig(bookieConf);
             fail("Bookie should not start with previous instance id.");
         } catch (BookieException.InvalidCookieException e) {
             assertTrue(
@@ -536,9 +536,9 @@ public class CookieTest extends BookKeeperClusterTestCase {
         }
 
         // Now format the Bookie and restart.
-        Bookie.format(bookieConf, false, true);
+        BookieImpl.format(bookieConf, false, true);
         // After bookie format bookie should be able to start again.
-        new Bookie(bookieConf);
+        validateConfig(bookieConf);
     }
 
     /**
@@ -547,10 +547,8 @@ public class CookieTest extends BookKeeperClusterTestCase {
      */
     @Test
     public void testV2data() throws Exception {
-        File journalDir = newV2JournalDirectory();
-        tmpDirs.add(journalDir);
-        File ledgerDir = newV2LedgerDirectory();
-        tmpDirs.add(ledgerDir);
+        File journalDir = initV2JournalDirectory(tmpDirs.createNew("bookie", "journal"));
+        File ledgerDir = initV2LedgerDirectory(tmpDirs.createNew("bookie", "ledger"));
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
@@ -558,11 +556,18 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         try {
-            Bookie b = new Bookie(conf);
+            BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
             fail("Shouldn't have been able to start");
-        } catch (BookieException.InvalidCookieException ice) {
+        } catch (IOException ioe) {
             // correct behaviour
-            assertTrue("wrong exception", ice.getCause().getMessage().contains("upgrade needed"));
+            assertTrue("wrong exception", ioe.getMessage().contains("upgrade needed"));
+        }
+        try {
+            BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
+            fail("Shouldn't have been able to start");
+        } catch (IOException ioe) {
+            // correct behaviour
+            assertTrue("wrong exception", ioe.getMessage().contains("upgrade needed"));
         }
     }
 
@@ -572,10 +577,8 @@ public class CookieTest extends BookKeeperClusterTestCase {
      */
     @Test
     public void testV1data() throws Exception {
-        File journalDir = newV1JournalDirectory();
-        tmpDirs.add(journalDir);
-        File ledgerDir = newV1LedgerDirectory();
-        tmpDirs.add(ledgerDir);
+        File journalDir = initV1JournalDirectory(tmpDirs.createNew("bookie", "journal"));
+        File ledgerDir = initV1LedgerDirectory(tmpDirs.createNew("bookie", "ledger"));
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
@@ -583,11 +586,19 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         try {
-            Bookie b = new Bookie(conf);
+            BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
             fail("Shouldn't have been able to start");
-        } catch (BookieException.InvalidCookieException ice) {
+        } catch (IOException ioe) {
             // correct behaviour
-            assertTrue("wrong exception", ice.getCause().getMessage().contains("upgrade needed"));
+            assertTrue("wrong exception", ioe.getMessage().contains("upgrade needed"));
+        }
+
+        try {
+            BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
+            fail("Shouldn't have been able to start");
+        } catch (IOException ioe) {
+            // correct behaviour
+            assertTrue("wrong exception", ioe.getMessage().contains("upgrade needed"));
         }
     }
 
@@ -605,13 +616,11 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setLedgerDirNames(ledgerDirs)
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
 
         conf.setUseHostNameAsBookieID(true);
         try {
-            new Bookie(conf);
+            validateConfig(conf);
             fail("Should not start a bookie with hostname if the bookie has been started with an ip");
         } catch (InvalidCookieException e) {
             // expected
@@ -632,13 +641,11 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         conf.setUseHostNameAsBookieID(false);
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
 
         conf.setAdvertisedAddress("unknown");
         try {
-            new Bookie(conf);
+            validateConfig(conf);
             fail("Should not start a bookie with ip if the bookie has been started with an ip");
         } catch (InvalidCookieException e) {
             // expected
@@ -659,13 +666,11 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         conf.setUseHostNameAsBookieID(true);
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
 
         conf.setUseHostNameAsBookieID(false);
         try {
-            new Bookie(conf);
+            validateConfig(conf);
             fail("Should not start a bookie with ip if the bookie has been started with an ip");
         } catch (InvalidCookieException e) {
             // expected
@@ -678,10 +683,8 @@ public class CookieTest extends BookKeeperClusterTestCase {
      */
     @Test
     public void testV2dataWithHostNameAsBookieID() throws Exception {
-        File journalDir = newV2JournalDirectory();
-        tmpDirs.add(journalDir);
-        File ledgerDir = newV2LedgerDirectory();
-        tmpDirs.add(ledgerDir);
+        File journalDir = initV2JournalDirectory(tmpDirs.createNew("bookie", "journal"));
+        File ledgerDir = initV2LedgerDirectory(tmpDirs.createNew("bookie", "ledger"));
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
@@ -689,13 +692,19 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         try {
-            conf.setUseHostNameAsBookieID(true);
-            new Bookie(conf);
+            BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
             fail("Shouldn't have been able to start");
-        } catch (BookieException.InvalidCookieException ice) {
+        } catch (IOException ioe) {
             // correct behaviour
-            assertTrue("wrong exception",
-                    ice.getCause().getMessage().contains("upgrade needed"));
+            assertTrue("wrong exception", ioe.getMessage().contains("upgrade needed"));
+        }
+
+        try {
+            BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
+            fail("Shouldn't have been able to start");
+        } catch (IOException ioe) {
+            // correct behaviour
+            assertTrue("wrong exception", ioe.getMessage().contains("upgrade needed"));
         }
     }
 
@@ -711,9 +720,7 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setLedgerDirNames(ledgerDirs)
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
         Versioned<Cookie> zkCookie = Cookie.readFromRegistrationManager(rm, conf);
         Version version1 = zkCookie.getVersion();
         assertTrue("Invalid type expected ZkVersion type",
@@ -743,9 +750,7 @@ public class CookieTest extends BookKeeperClusterTestCase {
             .setLedgerDirNames(ledgerDirs)
             .setBookiePort(bookiePort)
             .setMetadataServiceUri(zkUtil.getMetadataServiceUri());
-        Bookie b = new Bookie(conf); // should work fine
-        b.start();
-        b.shutdown();
+        validateConfig(conf);
         Versioned<Cookie> zkCookie = Cookie.readFromRegistrationManager(rm, conf);
         Cookie cookie = zkCookie.getValue();
         cookie.deleteFromRegistrationManager(rm, conf, zkCookie.getVersion());
