@@ -25,6 +25,7 @@ import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.meta.LedgerUnderreplicationManager;
 import org.apache.bookkeeper.replication.ReplicationException;
 import org.apache.bookkeeper.test.BookKeeperClusterTestCase;
+import org.apache.zookeeper.KeeperException;
 import org.junit.Assert;
 import org.junit.Test;
 
@@ -50,7 +51,7 @@ public class ForceAuditorChecksCmdTest extends BookKeeperClusterTestCase {
         String[] argv = new String[] { "forceauditchecks", "-calc", "-ppc", "-rc" };
         long curTime = System.currentTimeMillis();
 
-        final ServerConfiguration conf = bsConfs.get(0);
+        final ServerConfiguration conf = confByIndex(0);
         BookieShell bkShell = new BookieShell();
         bkShell.setConf(conf);
 
@@ -61,7 +62,7 @@ public class ForceAuditorChecksCmdTest extends BookKeeperClusterTestCase {
                 urM.setCheckAllLedgersCTime(curTime);
                 urM.setPlacementPolicyCheckCTime(curTime);
                 urM.setReplicasCheckCTime(curTime);
-            } catch (InterruptedException | ReplicationException e) {
+            } catch (InterruptedException | KeeperException | ReplicationException e) {
                 throw new UncheckedExecutionException(e);
             }
             return null;
@@ -86,7 +87,7 @@ public class ForceAuditorChecksCmdTest extends BookKeeperClusterTestCase {
                 if (replicasCheckCTime > (curTime - (20 * 24 * 60 * 60 * 1000))) {
                     Assert.fail("The replicasCheckCTime should have been reset to atleast 20 days old");
                 }
-            } catch (InterruptedException | ReplicationException e) {
+            } catch (InterruptedException | KeeperException | ReplicationException e) {
                 throw new UncheckedExecutionException(e);
             }
             return null;
