@@ -21,30 +21,25 @@
 package org.apache.bookkeeper.bookie.storage.ldb;
 
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
-
 import com.google.common.collect.Lists;
-
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufUtil;
 import io.netty.buffer.Unpooled;
-
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
-import java.nio.ByteBuffer;
 import java.util.List;
-
 import org.apache.bookkeeper.bookie.Bookie;
 import org.apache.bookkeeper.bookie.Bookie.NoEntryException;
 import org.apache.bookkeeper.bookie.BookieException;
-import org.apache.bookkeeper.bookie.CheckpointSource;
-import org.apache.bookkeeper.bookie.CheckpointSourceList;
+import org.apache.bookkeeper.bookie.BookieImpl;
 import org.apache.bookkeeper.bookie.EntryLocation;
 import org.apache.bookkeeper.bookie.EntryLogger;
 import org.apache.bookkeeper.bookie.LedgerDirsManager;
-import org.apache.bookkeeper.bookie.LogMark;
+import org.apache.bookkeeper.bookie.LedgerStorage;
+import org.apache.bookkeeper.bookie.TestBookieImpl;
 import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.conf.TestBKConfiguration;
 import org.apache.bookkeeper.proto.BookieProtocol;
@@ -52,29 +47,33 @@ import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
 
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
 /**
  * Unit test for {@link DbLedgerStorage}.
  */
 public class DbLedgerStorageTest {
-
+    private static final Logger log = LoggerFactory.getLogger(DbLedgerStorageTest.class);
     private DbLedgerStorage storage;
     private File tmpDir;
     private LedgerDirsManager ledgerDirsManager;
+    private ServerConfiguration conf;
 
     @Before
     public void setup() throws Exception {
         tmpDir = File.createTempFile("bkTest", ".dir");
         tmpDir.delete();
         tmpDir.mkdir();
-        File curDir = Bookie.getCurrentDirectory(tmpDir);
-        Bookie.checkDirectoryStructure(curDir);
+        File curDir = BookieImpl.getCurrentDirectory(tmpDir);
+        BookieImpl.checkDirectoryStructure(curDir);
 
         int gcWaitTime = 1000;
-        ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
+        conf = TestBKConfiguration.newServerConfiguration();
         conf.setGcWaitTime(gcWaitTime);
         conf.setLedgerStorageClass(DbLedgerStorage.class.getName());
         conf.setLedgerDirNames(new String[] { tmpDir.toString() });
-        Bookie bookie = new Bookie(conf);
+        BookieImpl bookie = new TestBookieImpl(conf);
 
         ledgerDirsManager = bookie.getLedgerDirsManager();
         storage = (DbLedgerStorage) bookie.getLedgerStorage();
@@ -140,6 +139,7 @@ public class DbLedgerStorageTest {
         assertEquals(true, ((DbLedgerStorage) storage).isFlushRequired());
 
         // Read from write cache
+        assertTrue(storage.entryExists(4, 1));
         ByteBuf res = storage.getEntry(4, 1);
         assertEquals(entry, res);
 
@@ -148,6 +148,7 @@ public class DbLedgerStorageTest {
         assertEquals(false, ((DbLedgerStorage) storage).isFlushRequired());
 
         // Read from db
+        assertTrue(storage.entryExists(4, 1));
         res = storage.getEntry(4, 1);
         assertEquals(entry, res);
 
@@ -197,22 +198,13 @@ public class DbLedgerStorageTest {
         storage.deleteLedger(4);
         assertEquals(false, storage.ledgerExists(4));
 
-        // Should not throw exception event if the ledger was deleted
-        storage.getEntry(4, 4);
-        assertEquals(3, storage.getLastAddConfirmed(4));
-
-        storage.addEntry(Unpooled.wrappedBuffer(entry2));
-        res = storage.getEntry(4, BookieProtocol.LAST_ADD_CONFIRMED);
-        assertEquals(entry4, res);
-        assertEquals(3, storage.getLastAddConfirmed(4));
-
-        // Get last entry from storage
+        // remove entries for ledger 4 from cache
         storage.flush();
 
         try {
             storage.getEntry(4, 4);
             fail("Should have thrown exception since the ledger was deleted");
-        } catch (NoEntryException e) {
+        } catch (Bookie.NoLedgerException e) {
             // ok
         }
     }
@@ -227,7 +219,6 @@ public class DbLedgerStorageTest {
         entry3.writeBytes("entry-3".getBytes());
         storage.addEntry(entry3);
 
-
         // Simulate bookie compaction
         SingleDirectoryDbLedgerStorage singleDirStorage = ((DbLedgerStorage) storage).getLedgerStorageList().get(0);
         EntryLogger entryLogger = singleDirStorage.getEntryLogger();
@@ -238,7 +229,6 @@ public class DbLedgerStorageTest {
         newEntry3.writeBytes("new-entry-3".getBytes());
         long location = entryLogger.addEntry(4L, newEntry3, false);
 
-        storage.flush();
         List<EntryLocation> locations = Lists.newArrayList(new EntryLocation(4, 3, location));
         singleDirStorage.updateEntriesLocations(locations);
 
@@ -261,7 +251,7 @@ public class DbLedgerStorageTest {
         conf.setLedgerDirNames(new String[] { firstDir.getCanonicalPath(), secondDir.getCanonicalPath() });
 
         // Should not fail
-        Bookie bookie = new Bookie(conf);
+        Bookie bookie = new TestBookieImpl(conf);
         assertEquals(2, ((DbLedgerStorage) bookie.getLedgerStorage()).getLedgerStorageList().size());
 
         bookie.shutdown();
@@ -445,187 +435,201 @@ public class DbLedgerStorageTest {
     }
 
     @Test
-    public void testGetLedgerDirsListeners() throws IOException {
-        // we should have two listeners, one is the SingleLedgerDirectories listener,
-        // and another is EntryLogManagerForEntryLogPerLedger
-        assertEquals(2, ledgerDirsManager.getListeners().size());
+    public void testLimboStateSucceedsWhenInLimboButHasEntry() throws Exception {
+        storage.setMasterKey(1, "foobar".getBytes());
+
+        ByteBuf entry0 = Unpooled.buffer(1024);
+        entry0.writeLong(1); // ledger id
+        entry0.writeLong(0); // entry id
+        entry0.writeBytes("entry-0".getBytes());
+
+        storage.addEntry(entry0);
+        storage.flush();
+        storage.setLimboState(1);
+
+        try {
+            storage.getEntry(1, 0);
+        } catch (BookieException.DataUnknownException e) {
+            fail("Should have been able to read entry");
+        }
+
+        storage.shutdown();
     }
 
     @Test
-    public void testMultiLedgerDirectoryCheckpoint() throws Exception {
-        int gcWaitTime = 1000;
-        File firstDir = new File(tmpDir, "dir1");
-        File secondDir = new File(tmpDir, "dir2");
-        ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
-        conf.setGcWaitTime(gcWaitTime);
-        conf.setProperty(DbLedgerStorage.WRITE_CACHE_MAX_SIZE_MB, 4);
-        conf.setProperty(DbLedgerStorage.READ_AHEAD_CACHE_MAX_SIZE_MB, 4);
-        conf.setLedgerStorageClass(DbLedgerStorage.class.getName());
-        conf.setLedgerDirNames(new String[] { firstDir.getCanonicalPath(), secondDir.getCanonicalPath() });
+    public void testLimboStateThrowsInLimboWhenNoEntry() throws Exception {
+        storage.setMasterKey(1, "foobar".getBytes());
 
-        Bookie bookie = new Bookie(conf);
-        ByteBuf entry1 = Unpooled.buffer(1024);
-        entry1.writeLong(1); // ledger id
-        entry1.writeLong(2); // entry id
-        entry1.writeBytes("entry-1".getBytes());
+        ByteBuf entry0 = Unpooled.buffer(1024);
+        entry0.writeLong(1); // ledger id
+        entry0.writeLong(0); // entry id
+        entry0.writeBytes("entry-0".getBytes());
 
-        bookie.getLedgerStorage().addEntry(entry1);
-        // write one entry to first ledger directory and flush with logMark(1, 2),
-        // only the first ledger directory should have lastMark
-        bookie.getJournals().get(0).getLastLogMark().getCurMark().setLogMark(1, 2);
-        ((DbLedgerStorage) bookie.getLedgerStorage()).getLedgerStorageList().get(0).flush();
-
-        File firstDirMark = new File(firstDir + "/current", "lastMark");
-        File secondDirMark = new File(secondDir + "/current", "lastMark");
-
-        // LedgerStorage flush won't trigger lastMark update due to two ledger directories configured
-        try {
-            readLogMark(firstDirMark);
-            readLogMark(secondDirMark);
-            fail();
-        } catch (Exception e) {
-            //
-        }
-
-        // write the second entry to second leger directory and flush with log(4, 5),
-        // the fist ledger directory's lastMark is (1, 2) and the second ledger directory's lastMark is (4, 5);
-        ByteBuf entry2 = Unpooled.buffer(1024);
-        entry2.writeLong(2); // ledger id
-        entry2.writeLong(1); // entry id
-        entry2.writeBytes("entry-2".getBytes());
-
-        bookie.getLedgerStorage().addEntry(entry2);
-        // write one entry to first ledger directory and flush with logMark(1, 2),
-        // only the first ledger directory should have lastMark
-        bookie.getJournals().get(0).getLastLogMark().getCurMark().setLogMark(4, 5);
-        ((DbLedgerStorage) bookie.getLedgerStorage()).getLedgerStorageList().get(1).flush();
-
-        // LedgerStorage flush won't trigger lastMark update due to two ledger directories configured
-        try {
-            readLogMark(firstDirMark);
-            readLogMark(secondDirMark);
-            fail();
-        } catch (Exception e) {
-            //
-        }
-
-        // The dbLedgerStorage flush also won't trigger lastMark update due to two ledger directories configured.
-        bookie.getLedgerStorage().flush();
-        try {
-            readLogMark(firstDirMark);
-            readLogMark(secondDirMark);
-            fail();
-        } catch (Exception e) {
-            //
-        }
-
-        // trigger checkpoint simulate SyncThread do checkpoint.
-        CheckpointSource checkpointSource = new CheckpointSourceList(bookie.getJournals());
-        bookie.getJournals().get(0).getLastLogMark().getCurMark().setLogMark(7, 8);
-        CheckpointSource.Checkpoint checkpoint = checkpointSource.newCheckpoint();
-        checkpointSource.checkpointComplete(checkpoint, false);
+        storage.addEntry(entry0);
+        storage.flush();
+        storage.setLimboState(1);
 
         try {
-            LogMark firstLogMark = readLogMark(firstDirMark);
-            LogMark secondLogMark = readLogMark(secondDirMark);
-            assertEquals(7, firstLogMark.getLogFileId());
-            assertEquals(8, firstLogMark.getLogFileOffset());
-            assertEquals(7, secondLogMark.getLogFileId());
-            assertEquals(8, secondLogMark.getLogFileOffset());
-        } catch (Exception e) {
-            fail();
+            storage.getEntry(1, 1);
+        } catch (NoEntryException nee) {
+            fail("Shouldn't have seen NoEntryException");
+        } catch (BookieException.DataUnknownException e) {
+            // expected
         }
 
-        // test replay journal lastMark, to make sure we get the right LastMark position
-        bookie.getJournals().get(0).getLastLogMark().readLog();
-        LogMark logMark = bookie.getJournals().get(0).getLastLogMark().getCurMark();
-        assertEquals(7, logMark.getLogFileId());
-        assertEquals(8, logMark.getLogFileOffset());
-    }
-
-    private LogMark readLogMark(File file) throws IOException {
-        byte[] buff = new byte[16];
-        ByteBuffer bb = ByteBuffer.wrap(buff);
-        LogMark mark = new LogMark();
-        try (FileInputStream fis = new FileInputStream(file)) {
-            int bytesRead = fis.read(buff);
-            if (bytesRead != 16) {
-                throw new IOException("Couldn't read enough bytes from lastMark."
-                    + " Wanted " + 16 + ", got " + bytesRead);
+        storage.shutdown();
+        Bookie restartedBookie = new TestBookieImpl(conf);
+        DbLedgerStorage restartedStorage = (DbLedgerStorage) restartedBookie.getLedgerStorage();
+        try {
+            try {
+                restartedStorage.getEntry(1, 1);
+            } catch (NoEntryException nee) {
+                fail("Shouldn't have seen NoEntryException");
+            } catch (BookieException.DataUnknownException e) {
+                // expected
             }
+        } finally {
+            restartedStorage.shutdown();
         }
-        bb.clear();
-        mark.readLogMark(bb);
-
-        return mark;
     }
 
     @Test
-    public void testSingleLedgerDirectoryCheckpoint() throws Exception {
-        int gcWaitTime = 1000;
-        File ledgerDir = new File(tmpDir, "dir");
-        ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
-        conf.setGcWaitTime(gcWaitTime);
-        conf.setProperty(DbLedgerStorage.WRITE_CACHE_MAX_SIZE_MB, 4);
-        conf.setProperty(DbLedgerStorage.READ_AHEAD_CACHE_MAX_SIZE_MB, 4);
-        conf.setLedgerStorageClass(DbLedgerStorage.class.getName());
-        conf.setLedgerDirNames(new String[] { ledgerDir.getCanonicalPath() });
+    public void testLimboStateThrowsNoEntryExceptionWhenLimboCleared() throws Exception {
+        storage.setMasterKey(1, "foobar".getBytes());
 
-        Bookie bookie = new Bookie(conf);
-        ByteBuf entry1 = Unpooled.buffer(1024);
-        entry1.writeLong(1); // ledger id
-        entry1.writeLong(2); // entry id
-        entry1.writeBytes("entry-1".getBytes());
-        bookie.getLedgerStorage().addEntry(entry1);
+        ByteBuf entry0 = Unpooled.buffer(1024);
+        entry0.writeLong(1); // ledger id
+        entry0.writeLong(0); // entry id
+        entry0.writeBytes("entry-0".getBytes());
 
-        bookie.getJournals().get(0).getLastLogMark().getCurMark().setLogMark(1, 2);
-        ((DbLedgerStorage) bookie.getLedgerStorage()).getLedgerStorageList().get(0).flush();
-
-        File ledgerDirMark = new File(ledgerDir + "/current", "lastMark");
-        try {
-            LogMark logMark = readLogMark(ledgerDirMark);
-            assertEquals(1, logMark.getLogFileId());
-            assertEquals(2, logMark.getLogFileOffset());
-        } catch (Exception e) {
-            fail();
-        }
-
-        ByteBuf entry2 = Unpooled.buffer(1024);
-        entry2.writeLong(2); // ledger id
-        entry2.writeLong(1); // entry id
-        entry2.writeBytes("entry-2".getBytes());
-
-        bookie.getLedgerStorage().addEntry(entry2);
-        // write one entry to first ledger directory and flush with logMark(1, 2),
-        // only the first ledger directory should have lastMark
-        bookie.getJournals().get(0).getLastLogMark().getCurMark().setLogMark(4, 5);
-
-        bookie.getLedgerStorage().flush();
-        try {
-            LogMark logMark = readLogMark(ledgerDirMark);
-            assertEquals(4, logMark.getLogFileId());
-            assertEquals(5, logMark.getLogFileOffset());
-        } catch (Exception e) {
-            fail();
-        }
-
-        CheckpointSource checkpointSource = new CheckpointSourceList(bookie.getJournals());
-        bookie.getJournals().get(0).getLastLogMark().getCurMark().setLogMark(7, 8);
-        CheckpointSource.Checkpoint checkpoint = checkpointSource.newCheckpoint();
-        checkpointSource.checkpointComplete(checkpoint, false);
+        storage.addEntry(entry0);
+        storage.flush();
+        storage.setLimboState(1);
 
         try {
-            LogMark firstLogMark = readLogMark(ledgerDirMark);
-            assertEquals(7, firstLogMark.getLogFileId());
-            assertEquals(8, firstLogMark.getLogFileOffset());
-        } catch (Exception e) {
-            fail();
+            storage.getEntry(1, 1);
+        } catch (NoEntryException nee) {
+            fail("Shouldn't have seen NoEntryException");
+        } catch (BookieException.DataUnknownException e) {
+            // expected
         }
 
-        // test replay journal lastMark, to make sure we get the right LastMark position
-        bookie.getJournals().get(0).getLastLogMark().readLog();
-        LogMark logMark = bookie.getJournals().get(0).getLastLogMark().getCurMark();
-        assertEquals(7, logMark.getLogFileId());
-        assertEquals(8, logMark.getLogFileOffset());
+        storage.clearLimboState(1);
+        try {
+            storage.getEntry(1, 1);
+        } catch (NoEntryException nee) {
+            // expected
+        } catch (BookieException.DataUnknownException e) {
+            fail("Should have seen NoEntryException");
+        }
+    }
+
+    @Test
+    public void testLimboStateSucceedsWhenFenced() throws Exception {
+        storage.setMasterKey(1, "foobar".getBytes());
+
+        ByteBuf entry0 = Unpooled.buffer(1024);
+        entry0.writeLong(1); // ledger id
+        entry0.writeLong(0); // entry id
+        entry0.writeBytes("entry-0".getBytes());
+
+        storage.addEntry(entry0);
+        storage.flush();
+        storage.setFenced(1);
+        storage.setLimboState(1);
+
+        try {
+            storage.isFenced(1);
+        } catch (IOException ioe) {
+            fail("Should have been able to get isFenced response");
+        }
+
+        storage.shutdown();
+    }
+
+    @Test
+    public void testLimboStateThrowsInLimboWhenNotFenced() throws Exception {
+        storage.setMasterKey(1, "foobar".getBytes());
+
+        ByteBuf entry0 = Unpooled.buffer(1024);
+        entry0.writeLong(1); // ledger id
+        entry0.writeLong(0); // entry id
+        entry0.writeBytes("entry-0".getBytes());
+
+        storage.addEntry(entry0);
+        storage.flush();
+        storage.setLimboState(1);
+
+        try {
+            storage.isFenced(1);
+            fail("Shouldn't have been able to get isFenced response");
+        } catch (BookieException.DataUnknownException e) {
+            // expected
+        }
+
+        storage.shutdown();
+    }
+
+    @Test
+    public void testHasEntry() throws Exception {
+        long ledgerId = 0xbeefee;
+        storage.setMasterKey(ledgerId, "foobar".getBytes());
+
+        ByteBuf entry0 = Unpooled.buffer(1024);
+        entry0.writeLong(ledgerId); // ledger id
+        entry0.writeLong(0); // entry id
+        entry0.writeBytes("entry-0".getBytes());
+
+        storage.addEntry(entry0);
+
+        // should come from write cache
+        assertTrue(storage.entryExists(ledgerId, 0));
+        assertFalse(storage.entryExists(ledgerId, 1));
+
+        storage.flush();
+
+        // should come from storage
+        assertTrue(storage.entryExists(ledgerId, 0));
+        assertFalse(storage.entryExists(ledgerId, 1));
+
+        // pull entry into readcache
+        storage.getEntry(ledgerId, 0);
+
+        // should come from read cache
+        assertTrue(storage.entryExists(ledgerId, 0));
+        assertFalse(storage.entryExists(ledgerId, 1));
+    }
+
+    @Test
+    public void testStorageStateFlags() throws Exception {
+        assertTrue(storage.getStorageStateFlags().isEmpty());
+
+        storage.setStorageStateFlag(LedgerStorage.StorageState.NEEDS_INTEGRITY_CHECK);
+        assertTrue(storage.getStorageStateFlags()
+                   .contains(LedgerStorage.StorageState.NEEDS_INTEGRITY_CHECK));
+
+        storage.shutdown();
+        Bookie restartedBookie1 = new TestBookieImpl(conf);
+        DbLedgerStorage restartedStorage1 = (DbLedgerStorage) restartedBookie1.getLedgerStorage();
+        try {
+            assertTrue(restartedStorage1.getStorageStateFlags()
+                   .contains(LedgerStorage.StorageState.NEEDS_INTEGRITY_CHECK));
+            restartedStorage1.clearStorageStateFlag(LedgerStorage.StorageState.NEEDS_INTEGRITY_CHECK);
+
+            assertFalse(restartedStorage1.getStorageStateFlags()
+                   .contains(LedgerStorage.StorageState.NEEDS_INTEGRITY_CHECK));
+
+        } finally {
+            restartedStorage1.shutdown();
+        }
+
+        Bookie restartedBookie2 = new TestBookieImpl(conf);
+        DbLedgerStorage restartedStorage2 = (DbLedgerStorage) restartedBookie2.getLedgerStorage();
+        try {
+            assertFalse(restartedStorage2.getStorageStateFlags()
+                   .contains(LedgerStorage.StorageState.NEEDS_INTEGRITY_CHECK));
+        } finally {
+            restartedStorage2.shutdown();
+        }
     }
 }
