@@ -26,23 +26,18 @@ import static org.apache.bookkeeper.client.BookKeeperClientStats.CLIENT_SCOPE;
 import static org.apache.bookkeeper.client.BookKeeperClientStats.READ_OP_DM;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
-
 import com.google.common.collect.Lists;
 import io.netty.buffer.AbstractByteBufAllocator;
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.PooledByteBufAllocator;
 import io.netty.buffer.Unpooled;
 import io.netty.buffer.UnpooledByteBufAllocator;
-
 import java.io.IOException;
-import java.net.URI;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.List;
@@ -54,9 +49,8 @@ import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
-
-import org.apache.bookkeeper.bookie.Bookie;
 import org.apache.bookkeeper.bookie.BookieException;
+import org.apache.bookkeeper.bookie.TestBookieImpl;
 import org.apache.bookkeeper.client.AsyncCallback.AddCallback;
 import org.apache.bookkeeper.client.BKException.BKLedgerClosedException;
 import org.apache.bookkeeper.client.BookKeeper.DigestType;
@@ -64,21 +58,12 @@ import org.apache.bookkeeper.client.api.LedgerEntries;
 import org.apache.bookkeeper.client.api.ReadHandle;
 import org.apache.bookkeeper.client.api.WriteAdvHandle;
 import org.apache.bookkeeper.conf.ServerConfiguration;
-import org.apache.bookkeeper.meta.LedgerManagerFactory;
-import org.apache.bookkeeper.meta.LedgerUnderreplicationManager;
+import org.apache.bookkeeper.meta.LedgerMetadataSerDe;
 import org.apache.bookkeeper.meta.LongHierarchicalLedgerManagerFactory;
-import org.apache.bookkeeper.meta.MetadataBookieDriver;
-import org.apache.bookkeeper.meta.MetadataDrivers;
-import org.apache.bookkeeper.meta.zk.ZKMetadataDriverBase;
 import org.apache.bookkeeper.net.BookieId;
-import org.apache.bookkeeper.replication.ReplicationTestUtil;
-import org.apache.bookkeeper.replication.ReplicationWorker;
-import org.apache.bookkeeper.stats.NullStatsLogger;
 import org.apache.bookkeeper.test.BookKeeperClusterTestCase;
-import org.apache.bookkeeper.util.BookKeeperConstants;
+import org.apache.bookkeeper.test.TestStatsProvider;
 import org.apache.commons.lang3.tuple.Pair;
-import org.apache.zookeeper.KeeperException;
-import org.awaitility.Awaitility;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
@@ -258,8 +243,7 @@ public class BookieWriteLedgerTest extends
         // Replace the bookie with a fake bookie
         ServerConfiguration conf = killBookie(ensemble.get(0));
         BookieWriteLedgerTest.CorruptReadBookie corruptBookie = new BookieWriteLedgerTest.CorruptReadBookie(conf);
-        bs.add(startBookie(conf, corruptBookie));
-        bsConfs.add(conf);
+        startAndAddBookie(conf, corruptBookie);
 
         i = numEntriesToWrite;
         numEntriesToWrite = numEntriesToWrite + 50;
@@ -747,7 +731,7 @@ public class BookieWriteLedgerTest extends
                         .withDigestType(org.apache.bookkeeper.client.api.DigestType.CRC32)
                         .withPassword(ledgerPassword).makeAdv().withLedgerId(ledgerId)
                         .execute()
-                        .thenCompose(writer -> { // Add entries to ledger when created
+                        .thenApply(writer -> { // Add entries to ledger when created
                                 LOG.info("Writing stream of {} entries to {}",
                                          numEntriesToWrite, ledgerId);
                                 List<ByteBuf> entries = rng.ints(numEntriesToWrite, 0, maxInt)
@@ -766,8 +750,8 @@ public class BookieWriteLedgerTest extends
                                              ledgerId, entryId, entry.slice().readInt());
                                     lastRequest = writer.writeAsync(entryId, entry);
                                 }
-                                return lastRequest
-                                        .thenApply(___ -> Pair.of(writer, entries));
+                                lastRequest.join();
+                                return Pair.of(writer, entries);
                             });
                 })
             .parallel().map(CompletableFuture::join) // wait for all creations and adds in parallel
@@ -1442,62 +1426,13 @@ public class BookieWriteLedgerTest extends
     }
 
     @Test
-    public void testReadLacNotSameWithMetadataLedgerReplication() throws Exception {
-       lh = bkc.createLedger(3, 3, 2, digestType, ledgerPassword);
-        for (int i = 0; i < 10; ++i) {
-            ByteBuffer entry = ByteBuffer.allocate(4);
-            entry.putInt(rng.nextInt(maxInt));
-            entry.position(0);
-            lh.addEntry(entry.array());
-        }
-
-        List<BookieId> ensemble = lh.getLedgerMetadata().getAllEnsembles().entrySet().iterator().next().getValue();
-        assertEquals(1, lh.getLedgerMetadata().getAllEnsembles().size());
-        killBookie(ensemble.get(1));
-
-        try {
-            lh.ensembleChangeLoop(ensemble, Collections.singletonMap(1, ensemble.get(1)));
-        } catch (Exception e) {
-            fail();
-        }
-
-        LedgerHandle lh1 = bkc.openLedgerNoRecovery(lh.ledgerId, digestType, ledgerPassword);
-        assertEquals(2, lh1.getLedgerMetadata().getAllEnsembles().size());
-        List<BookieId> firstEnsemble = lh1.getLedgerMetadata().getAllEnsembles().firstEntry().getValue();
-
-        long entryId = lh1.getLedgerMetadata().getAllEnsembles().lastEntry().getKey() - 1;
-        try {
-            lh1.readAsync(entryId, entryId).get();
-            fail();
-        } catch (Exception e) {
-            LOG.info("Failed to read entry: {} ", entryId, e);
-        }
-
-        MetadataBookieDriver driver = MetadataDrivers.getBookieDriver(
-            URI.create(baseConf.getMetadataServiceUri()));
-        driver.initialize(baseConf, ()-> {}, NullStatsLogger.INSTANCE);
-        // initialize urReplicationManager
-        LedgerManagerFactory mFactory = driver.getLedgerManagerFactory();
-        LedgerUnderreplicationManager underReplicationManager = mFactory.newLedgerUnderreplicationManager();
-        baseConf.setOpenLedgerRereplicationGracePeriod(String.valueOf(30));
-
-
-        ReplicationWorker replicationWorker = new ReplicationWorker(baseConf);
-        replicationWorker.start();
-        String basePath = ZKMetadataDriverBase.resolveZkLedgersRootPath(baseClientConf) + '/'
-            + BookKeeperConstants.UNDER_REPLICATION_NODE
-            + BookKeeperConstants.DEFAULT_ZK_LEDGERS_ROOT_PATH;
-
-        try {
-            underReplicationManager.markLedgerUnderreplicated(lh1.getId(), ensemble.get(1).toString());
-            Awaitility.waitAtMost(30, TimeUnit.SECONDS).untilAsserted(() ->
-                assertFalse(ReplicationTestUtil.isLedgerInUnderReplication(zkc, lh1.getId(), basePath))
-            );
-
-            assertNotEquals(firstEnsemble, lh1.getLedgerMetadata().getAllEnsembles().firstEntry().getValue());
-        } finally {
-            replicationWorker.shutdown();
-        }
+    public void testLedgerMetadataTest() throws Exception {
+        baseClientConf.setLedgerMetadataFormatVersion(LedgerMetadataSerDe.METADATA_FORMAT_VERSION_2);
+        BookKeeperTestClient bkc = new BookKeeperTestClient(baseClientConf, new TestStatsProvider());
+        // Create a ledger
+        lh = bkc.createLedger(3, 3, 2, digestType, ledgerPassword);
+        assertEquals(lh.getLedgerMetadata().getMetadataFormatVersion(), LedgerMetadataSerDe.METADATA_FORMAT_VERSION_2);
+        lh.close();
     }
 
     private void readEntries(LedgerHandle lh, List<byte[]> entries) throws InterruptedException, BKException {
@@ -1559,18 +1494,18 @@ public class BookieWriteLedgerTest extends
         }
     }
 
-    static class CorruptReadBookie extends Bookie {
+    static class CorruptReadBookie extends TestBookieImpl {
 
         static final Logger LOG = LoggerFactory.getLogger(CorruptReadBookie.class);
         ByteBuf localBuf;
 
         public CorruptReadBookie(ServerConfiguration conf)
-                throws IOException, KeeperException, InterruptedException, BookieException {
+                throws Exception {
             super(conf);
         }
 
         @Override
-        public ByteBuf readEntry(long ledgerId, long entryId) throws IOException, NoLedgerException {
+        public ByteBuf readEntry(long ledgerId, long entryId) throws IOException, NoLedgerException, BookieException {
             localBuf = super.readEntry(ledgerId, entryId);
 
             int capacity = 0;
