@@ -72,7 +72,6 @@ public class WriteEntryProcessorTest {
         when(requestProcessor.getBookie()).thenReturn(bookie);
         when(requestProcessor.getRequestStats()).thenReturn(new RequestStats(NullStatsLogger.INSTANCE));
         when(channel.isActive()).thenReturn(true);
-        when(channel.isWritable()).thenReturn(true);
         processor = WriteEntryProcessor.create(
             request,
             channel,
@@ -100,9 +99,7 @@ public class WriteEntryProcessorTest {
     @Test
     public void testNoneHighPriorityWritesOnReadOnlyBookie() throws Exception {
         when(bookie.isReadOnly()).thenReturn(true);
-        ChannelPromise mockPromise = mock(ChannelPromise.class);
-        when(channel.newPromise()).thenReturn(mockPromise);
-        when(mockPromise.addListener(any())).thenReturn(mockPromise);
+        when(channel.voidPromise()).thenReturn(mock(ChannelPromise.class));
 
         AtomicReference<Object> writtenObject = new AtomicReference<>();
         CountDownLatch latch = new CountDownLatch(1);
@@ -132,9 +129,7 @@ public class WriteEntryProcessorTest {
 
         when(bookie.isReadOnly()).thenReturn(true);
         when(bookie.isAvailableForHighPriorityWrites()).thenReturn(false);
-        ChannelPromise mockPromise = mock(ChannelPromise.class);
-        when(channel.newPromise()).thenReturn(mockPromise);
-        when(mockPromise.addListener(any())).thenReturn(mockPromise);
+        when(channel.voidPromise()).thenReturn(mock(ChannelPromise.class));
 
         AtomicReference<Object> writtenObject = new AtomicReference<>();
         CountDownLatch latch = new CountDownLatch(1);
@@ -164,9 +159,7 @@ public class WriteEntryProcessorTest {
 
         when(bookie.isReadOnly()).thenReturn(true);
         when(bookie.isAvailableForHighPriorityWrites()).thenReturn(true);
-        ChannelPromise mockPromise = mock(ChannelPromise.class);
-        when(channel.newPromise()).thenReturn(mockPromise);
-        when(mockPromise.addListener(any())).thenReturn(mockPromise);
+        when(channel.voidPromise()).thenReturn(mock(ChannelPromise.class));
         doAnswer(invocationOnMock -> {
             processor.writeComplete(0, request.ledgerId, request.entryId, null, null);
             return null;
@@ -199,9 +192,7 @@ public class WriteEntryProcessorTest {
     @Test
     public void testNormalWritesOnWritableBookie() throws Exception {
         when(bookie.isReadOnly()).thenReturn(false);
-        ChannelPromise mockPromise = mock(ChannelPromise.class);
-        when(channel.newPromise()).thenReturn(mockPromise);
-        when(mockPromise.addListener(any())).thenReturn(mockPromise);
+        when(channel.voidPromise()).thenReturn(mock(ChannelPromise.class));
         doAnswer(invocationOnMock -> {
             processor.writeComplete(0, request.ledgerId, request.entryId, null, null);
             return null;
@@ -234,9 +225,7 @@ public class WriteEntryProcessorTest {
     @Test
     public void testWritesCacheFlushTimeout() throws Exception {
         when(bookie.isReadOnly()).thenReturn(false);
-        ChannelPromise mockPromise = mock(ChannelPromise.class);
-        when(channel.newPromise()).thenReturn(mockPromise);
-        when(mockPromise.addListener(any())).thenReturn(mockPromise);
+        when(channel.voidPromise()).thenReturn(mock(ChannelPromise.class));
         when(channel.writeAndFlush(any())).thenReturn(mock(ChannelPromise.class));
         doAnswer(invocationOnMock -> {
             throw new BookieException.OperationRejectedException();
