@@ -34,10 +34,11 @@ import java.util.Map.Entry;
 import java.util.Optional;
 import java.util.TimerTask;
 import java.util.concurrent.CopyOnWriteArrayList;
-import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
+
 import lombok.Cleanup;
+
 import org.apache.bookkeeper.client.BKException;
 import org.apache.bookkeeper.client.BookKeeper;
 import org.apache.bookkeeper.client.ClientUtil;
@@ -66,7 +67,6 @@ import org.apache.bookkeeper.util.BookKeeperConstants;
 import org.apache.bookkeeper.zookeeper.BoundExponentialBackoffRetryPolicy;
 import org.apache.bookkeeper.zookeeper.ZooKeeperClient;
 import org.apache.bookkeeper.zookeeper.ZooKeeperWatcherBase;
-import org.apache.commons.lang3.reflect.FieldUtils;
 import org.apache.zookeeper.AsyncCallback.StatCallback;
 import org.apache.zookeeper.KeeperException;
 import org.apache.zookeeper.WatchedEvent;
@@ -75,7 +75,6 @@ import org.apache.zookeeper.Watcher.Event.EventType;
 import org.apache.zookeeper.Watcher.Event.KeeperState;
 import org.apache.zookeeper.ZooKeeper;
 import org.apache.zookeeper.ZooKeeper.States;
-import org.awaitility.Awaitility;
 import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -104,16 +103,13 @@ public class TestReplicationWorker extends BookKeeperClusterTestCase {
     }
 
     TestReplicationWorker(String ledgerManagerFactory) {
-        super(3, 300);
+        super(3);
         LOG.info("Running test case using ledger manager : "
                 + ledgerManagerFactory);
         // set ledger manager name
         baseConf.setLedgerManagerFactoryClassName(ledgerManagerFactory);
         baseClientConf.setLedgerManagerFactoryClassName(ledgerManagerFactory);
         baseConf.setRereplicationEntryBatchSize(3);
-        baseConf.setZkTimeout(7000);
-        baseConf.setZkRetryBackoffMaxMs(500);
-        baseConf.setZkRetryBackoffStartMs(10);
     }
 
     @Override
@@ -137,7 +133,6 @@ public class TestReplicationWorker extends BookKeeperClusterTestCase {
             URI.create(baseConf.getMetadataServiceUri()));
         this.driver.initialize(
             baseConf,
-            () -> {},
             NullStatsLogger.INSTANCE);
         // initialize urReplicationManager
         mFactory = driver.getLedgerManagerFactory();
@@ -239,8 +234,7 @@ public class TestReplicationWorker extends BookKeeperClusterTestCase {
                 Thread.sleep(100);
             }
             // restart killed bookie
-            bs.add(startBookie(killedBookieConfig));
-            bsConfs.add(killedBookieConfig);
+            startAndAddBookie(killedBookieConfig);
             while (ReplicationTestUtil.isLedgerInUnderReplication(zkc, lh
                     .getId(), basePath)) {
                 Thread.sleep(100);
@@ -294,8 +288,7 @@ public class TestReplicationWorker extends BookKeeperClusterTestCase {
                 Thread.sleep(100);
             }
             // restart killed bookie
-            bs.add(startBookie(killedBookieConfig));
-            bsConfs.add(killedBookieConfig);
+            startAndAddBookie(killedBookieConfig);
             while (ReplicationTestUtil.isLedgerInUnderReplication(zkc, lh
                     .getId(), basePath)) {
                 Thread.sleep(100);
@@ -601,11 +594,9 @@ public class TestReplicationWorker extends BookKeeperClusterTestCase {
             int rw1UnableToReadEntriesForReplication = rw1.unableToReadEntriesForReplication.get(lh.getId()).size();
             int rw2UnableToReadEntriesForReplication = rw2.unableToReadEntriesForReplication.get(lh.getId()).size();
             assertTrue(
-                    "unableToReadEntriesForReplication in RW1: " + rw1UnableToReadEntriesForReplication
-                            + " in RW2: "
+                    "unableToReadEntriesForReplication in RW1: " + rw1UnableToReadEntriesForReplication + " in RW2: "
                             + rw2UnableToReadEntriesForReplication,
-                    (rw1UnableToReadEntriesForReplication == 0)
-                            || (rw2UnableToReadEntriesForReplication == 0));
+                    (rw1UnableToReadEntriesForReplication == 0) || (rw2UnableToReadEntriesForReplication == 0));
         } finally {
             rw1.shutdown();
             rw2.shutdown();
@@ -618,8 +609,7 @@ public class TestReplicationWorker extends BookKeeperClusterTestCase {
 
         public InjectedReplicationWorker(ServerConfiguration conf, StatsLogger statsLogger,
                 CopyOnWriteArrayList<Long> delayReplicationPeriods)
-                throws CompatibilityException, ReplicationException.UnavailableException,
-                InterruptedException, IOException {
+                throws CompatibilityException, KeeperException, InterruptedException, IOException {
             super(conf, statsLogger);
             this.delayReplicationPeriods = delayReplicationPeriods;
         }
@@ -839,66 +829,20 @@ public class TestReplicationWorker extends BookKeeperClusterTestCase {
             assertTrue("Replication worker should be running", rw.isRunning());
 
             stopZKCluster();
-            // ZK is down for shorter period than reconnect timeout
-            Thread.sleep(1000);
+            // Wait for disconnection to be picked up
+            for (int i = 0; i < 10; i++) {
+                if (!zk.getState().isConnected()) {
+                    break;
+                }
+                Thread.sleep(1000);
+            }
+            assertFalse(zk.getState().isConnected());
             startZKCluster();
 
-            assertTrue("Replication worker should not shutdown", rw.isRunning());
+            assertTrue("Replication worker should still be running", rw.isRunning());
         }
     }
 
-    /**
-     * Test that the replication worker shuts down on non-recoverable ZK connection loss.
-     */
-    @Test
-    public void testRWZKConnectionLostOnNonRecoverableZkError() throws Exception {
-        for (int j = 0; j < 3; j++) {
-            LedgerHandle lh = bkc.createLedger(1, 1, 1,
-                    BookKeeper.DigestType.CRC32, TESTPASSWD,
-                    null);
-            final long createdLedgerId = lh.getId();
-            for (int i = 0; i < 10; i++) {
-                lh.addEntry(data);
-            }
-            lh.close();
-        }
-
-        killBookie(2);
-        killBookie(1);
-        startNewBookie();
-        startNewBookie();
-
-        bsConfs.get(0).setRwRereplicateBackoffMs(100);
-        setAutoRecoveryEnabled(true);
-        startReplicationService();
-
-        Auditor auditor = getAuditor(10, TimeUnit.SECONDS);
-        ReplicationWorker rw = getAutoRecovery(bs.get(0)).replicationWorker;
-
-        ZkLedgerUnderreplicationManager ledgerUnderreplicationManager =
-                (ZkLedgerUnderreplicationManager) FieldUtils.readField(auditor,
-                        "ledgerUnderreplicationManager", true);
-
-        ZooKeeper zkc = (ZooKeeper) FieldUtils.readField(ledgerUnderreplicationManager, "zkc", true);
-        auditor.submitAuditTask().get();
-
-        assertTrue(zkc.getState().isConnected());
-        zkc.close();
-        assertFalse(zkc.getState().isConnected());
-
-        auditor.submitAuditTask();
-        rw.run();
-
-        for (int i = 0; i < 10; i++) {
-            if (!rw.isRunning() && !auditor.isRunning()) {
-                break;
-            }
-            Thread.sleep(1000);
-        }
-        assertFalse("Replication worker should NOT be running", rw.isRunning());
-        assertFalse("Auditor should NOT be running", auditor.isRunning());
-    }
-
     private void killAllBookies(LedgerHandle lh, BookieId excludeBK)
             throws Exception {
         // Killing all bookies except newly replicated bookie
@@ -1031,8 +975,7 @@ public class TestReplicationWorker extends BookKeeperClusterTestCase {
          */
         BookKeeper bkWithMockZK = new BookKeeper(baseClientConf, zkFaultInjectionWrapper);
         long ledgerId = 567L;
-        LedgerHandle lh = bkWithMockZK.createLedgerAdv(ledgerId, 2, 2, 2,
-                BookKeeper.DigestType.CRC32, TESTPASSWD,
+        LedgerHandle lh = bkWithMockZK.createLedgerAdv(ledgerId, 2, 2, 2, BookKeeper.DigestType.CRC32, TESTPASSWD,
                 null);
         for (int i = 0; i < 10; i++) {
             lh.addEntry(i, data);
@@ -1063,7 +1006,7 @@ public class TestReplicationWorker extends BookKeeperClusterTestCase {
          * 2, we should be able to replicate to the other bookie.
          */
         BookieId replicaToKill = lh.getLedgerMetadata().getAllEnsembles().get(0L).get(0);
-        LOG.info("Killing Bookie", replicaToKill);
+        LOG.info("Killing Bookie id {}", replicaToKill);
         killBookie(replicaToKill);
 
         /*
@@ -1134,30 +1077,4 @@ public class TestReplicationWorker extends BookKeeperClusterTestCase {
             bkWithMockZK.close();
         }
     }
-    @Test
-    public void testReplicateEmptyOpenStateLedger() throws Exception {
-        LedgerHandle lh = bkc.createLedger(3, 3, 2, BookKeeper.DigestType.CRC32, TESTPASSWD);
-        assertFalse(lh.getLedgerMetadata().isClosed());
-
-        List<BookieId> firstEnsemble = lh.getLedgerMetadata().getAllEnsembles().firstEntry().getValue();
-        List<BookieId> ensemble = lh.getLedgerMetadata().getAllEnsembles().entrySet().iterator().next().getValue();
-        killBookie(ensemble.get(1));
-
-        startNewBookie();
-        baseConf.setOpenLedgerRereplicationGracePeriod(String.valueOf(30));
-        ReplicationWorker replicationWorker = new ReplicationWorker(baseConf);
-        replicationWorker.start();
-
-        try {
-            underReplicationManager.markLedgerUnderreplicated(lh.getId(), ensemble.get(1).toString());
-            Awaitility.waitAtMost(60, TimeUnit.SECONDS).untilAsserted(() ->
-                assertFalse(ReplicationTestUtil.isLedgerInUnderReplication(zkc, lh.getId(), basePath))
-            );
-
-            LedgerHandle lh1 = bkc.openLedgerNoRecovery(lh.getId(), BookKeeper.DigestType.CRC32, TESTPASSWD);
-            assertTrue(lh1.getLedgerMetadata().isClosed());
-        } finally {
-            replicationWorker.shutdown();
-        }
-    }
 }
