@@ -21,37 +21,51 @@
 
 package org.apache.bookkeeper.test;
 
-
-
+import static org.apache.bookkeeper.bookie.BookKeeperServerStats.BOOKIE_SCOPE;
+import static org.apache.bookkeeper.bookie.BookKeeperServerStats.LD_INDEX_SCOPE;
+import static org.apache.bookkeeper.bookie.BookKeeperServerStats.LD_LEDGER_SCOPE;
 import static org.apache.bookkeeper.util.BookKeeperConstants.AVAILABLE_NODE;
-import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.assertFalse;
 
 import com.google.common.base.Stopwatch;
-import io.netty.buffer.ByteBufAllocator;
-
 import java.io.File;
 import java.io.IOException;
 import java.net.UnknownHostException;
-import java.util.HashMap;
+import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.LinkedList;
 import java.util.List;
-import java.util.Map;
-import java.util.Map.Entry;
+import java.util.Optional;
+import java.util.OptionalInt;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.Future;
 import java.util.concurrent.SynchronousQueue;
 import java.util.concurrent.TimeUnit;
-import java.util.function.Supplier;
-
+import java.util.function.Function;
+import java.util.stream.Collectors;
 import org.apache.bookkeeper.bookie.Bookie;
 import org.apache.bookkeeper.bookie.BookieException;
+import org.apache.bookkeeper.bookie.BookieImpl;
+import org.apache.bookkeeper.bookie.BookieResources;
+import org.apache.bookkeeper.bookie.LedgerDirsManager;
+import org.apache.bookkeeper.bookie.LedgerStorage;
+import org.apache.bookkeeper.bookie.LegacyCookieValidation;
+import org.apache.bookkeeper.bookie.MockUncleanShutdownDetection;
+import org.apache.bookkeeper.bookie.ReadOnlyBookie;
+import org.apache.bookkeeper.bookie.UncleanShutdownDetection;
+import org.apache.bookkeeper.bookie.UncleanShutdownDetectionImpl;
 import org.apache.bookkeeper.client.BookKeeperTestClient;
+import org.apache.bookkeeper.common.allocator.ByteBufAllocatorWithOomHandler;
 import org.apache.bookkeeper.common.allocator.PoolingPolicy;
 import org.apache.bookkeeper.conf.AbstractConfiguration;
 import org.apache.bookkeeper.conf.ClientConfiguration;
 import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.conf.TestBKConfiguration;
 import org.apache.bookkeeper.discover.BookieServiceInfo;
+import org.apache.bookkeeper.discover.RegistrationManager;
+import org.apache.bookkeeper.meta.LedgerManager;
+import org.apache.bookkeeper.meta.LedgerManagerFactory;
+import org.apache.bookkeeper.meta.MetadataBookieDriver;
 import org.apache.bookkeeper.meta.zk.ZKMetadataDriverBase;
 import org.apache.bookkeeper.metastore.InMemoryMetaStore;
 import org.apache.bookkeeper.net.BookieId;
@@ -59,12 +73,11 @@ import org.apache.bookkeeper.net.BookieSocketAddress;
 import org.apache.bookkeeper.proto.BookieServer;
 import org.apache.bookkeeper.replication.Auditor;
 import org.apache.bookkeeper.replication.AutoRecoveryMain;
-import org.apache.bookkeeper.replication.ReplicationException.CompatibilityException;
-import org.apache.bookkeeper.replication.ReplicationException.UnavailableException;
 import org.apache.bookkeeper.replication.ReplicationWorker;
-import org.apache.bookkeeper.util.IOUtils;
+import org.apache.bookkeeper.server.Main;
+import org.apache.bookkeeper.stats.StatsLogger;
+import org.apache.bookkeeper.util.DiskChecker;
 import org.apache.bookkeeper.util.PortManager;
-import org.apache.commons.io.FileUtils;
 import org.apache.zookeeper.KeeperException;
 import org.apache.zookeeper.ZooKeeper;
 import org.junit.After;
@@ -94,10 +107,8 @@ public abstract class BookKeeperClusterTestCase {
     protected String metadataServiceUri;
 
     // BookKeeper related variables
-    protected final List<File> tmpDirs = new LinkedList<File>();
-    protected final List<BookieServer> bs = new LinkedList<BookieServer>();
-    protected final List<ServerConfiguration> bsConfs = new LinkedList<ServerConfiguration>();
-    private final Map<BookieId, TestStatsProvider> bsLoggers = new HashMap<>();
+    protected final TmpDirs tmpDirs = new TmpDirs();
+    private final List<ServerTester> servers = new LinkedList<>();
 
     protected int numBookies;
     protected BookKeeperTestClient bkc;
@@ -110,8 +121,7 @@ public abstract class BookKeeperClusterTestCase {
      */
     protected final ServerConfiguration baseConf = TestBKConfiguration.newServerConfiguration();
     protected final ClientConfiguration baseClientConf = TestBKConfiguration.newClientConfiguration();
-
-    private final Map<BookieServer, AutoRecoveryMain> autoRecoveryProcesses = new HashMap<>();
+    private final ByteBufAllocatorWithOomHandler allocator = BookieResources.createAllocator(baseConf);
 
     private boolean isAutoRecoveryEnabled;
 
@@ -120,7 +130,7 @@ public abstract class BookKeeperClusterTestCase {
         try {
             c.run();
         } catch (Throwable e) {
-            LOG.error("Captured error: {}", e);
+            LOG.error("Captured error: ", e);
             asyncExceptions.add(e);
         }
     }
@@ -181,10 +191,10 @@ public abstract class BookKeeperClusterTestCase {
     public void tearDown() throws Exception {
         boolean failed = false;
         for (Throwable e : asyncExceptions) {
-            LOG.error("Got async exception: {}", e);
+            LOG.error("Got async exception: ", e);
             failed = true;
         }
-        assertTrue("Async failure", !failed);
+        assertFalse("Async failure", failed);
         Stopwatch sw = Stopwatch.createStarted();
         LOG.info("TearDown");
         Exception tearDownException = null;
@@ -204,7 +214,7 @@ public abstract class BookKeeperClusterTestCase {
         }
         // cleanup temp dirs
         try {
-            cleanupTempDirs();
+            tmpDirs.cleanup();
         } catch (Exception e) {
             LOG.error("Got Exception while trying to cleanupTempDirs", e);
             tearDownException = e;
@@ -215,12 +225,6 @@ public abstract class BookKeeperClusterTestCase {
         }
     }
 
-    protected File createTempDir(String prefix, String suffix) throws IOException {
-        File dir = IOUtils.createTempDir(prefix, suffix);
-        tmpDirs.add(dir);
-        return dir;
-    }
-
     /**
      * Start zookeeper cluster.
      *
@@ -271,27 +275,14 @@ public abstract class BookKeeperClusterTestCase {
             bkc.close();
         }
 
-        for (BookieServer server : bs) {
-            server.shutdown();
-            AutoRecoveryMain autoRecovery = autoRecoveryProcesses.get(server);
-            if (autoRecovery != null && isAutoRecoveryEnabled()) {
-                autoRecovery.shutdown();
-                LOG.debug("Shutdown auto recovery for bookieserver:"
-                        + server.getBookieId());
-            }
-        }
-        bs.clear();
-        bsLoggers.clear();
-    }
-
-    protected void cleanupTempDirs() throws Exception {
-        for (File f : tmpDirs) {
-            FileUtils.deleteDirectory(f);
+        for (ServerTester t : servers) {
+            t.shutdown();
         }
+        servers.clear();
     }
 
     protected ServerConfiguration newServerConfiguration() throws Exception {
-        File f = createTempDir("bookie", "test");
+        File f = tmpDirs.createNew("bookie", "test");
 
         int port;
         if (baseConf.isEnableLocalTransport() || !baseConf.getAllowEphemeralPorts()) {
@@ -325,23 +316,16 @@ public abstract class BookKeeperClusterTestCase {
     }
 
     protected void stopAllBookies(boolean shutdownClient) throws Exception {
-        for (BookieServer server : bs) {
-            server.shutdown();
+        for (ServerTester t : servers) {
+            t.shutdown();
         }
-        bsConfs.clear();
-        bs.clear();
+        servers.clear();
         if (shutdownClient && bkc != null) {
             bkc.close();
             bkc = null;
         }
     }
 
-    protected void startAllBookies() throws Exception {
-        for (ServerConfiguration conf : bsConfs) {
-            bs.add(startBookie(conf));
-        }
-    }
-
     protected String newMetadataServiceUri(String ledgersRootPath) {
         return zkUtil.getMetadataServiceUri(ledgersRootPath);
     }
@@ -354,32 +338,81 @@ public abstract class BookKeeperClusterTestCase {
      * Get bookie address for bookie at index.
      */
     public BookieId getBookie(int index) throws Exception {
-        if (bs.size() <= index || index < 0) {
-            throw new IllegalArgumentException("Invalid index, there are only " + bs.size()
-                                               + " bookies. Asked for " + index);
-        }
-        return bs.get(index).getBookieId();
+        return servers.get(index).getServer().getBookieId();
     }
 
-    public BookieSocketAddress getBookieAddress(int index) throws Exception {
-        return bkc.getBookieAddressResolver().resolve(getBookie(index));
+    protected List<BookieId> bookieAddresses() throws Exception {
+        List<BookieId> bookieIds = new ArrayList<>();
+        for (ServerTester a : servers) {
+            bookieIds.add(a.getServer().getBookieId());
+        }
+        return bookieIds;
+    }
+
+    protected List<File> bookieLedgerDirs() throws Exception {
+        return servers.stream()
+            .flatMap(t -> Arrays.stream(t.getConfiguration().getLedgerDirs()))
+            .collect(Collectors.toList());
+    }
+
+    protected List<File> bookieJournalDirs() throws Exception {
+        return servers.stream()
+            .flatMap(t -> Arrays.stream(t.getConfiguration().getJournalDirs()))
+            .collect(Collectors.toList());
+    }
+
+    protected BookieId addressByIndex(int index) throws Exception {
+        return servers.get(index).getServer().getBookieId();
+    }
+
+    protected BookieServer serverByIndex(int index) throws Exception {
+        return servers.get(index).getServer();
+    }
+
+    protected ServerConfiguration confByIndex(int index) throws Exception {
+        return servers.get(index).getConfiguration();
+    }
+
+    private Optional<ServerTester> byAddress(BookieId addr) throws UnknownHostException {
+        for (ServerTester s : servers) {
+            if (s.getServer().getBookieId().equals(addr)) {
+                return Optional.of(s);
+            }
+        }
+        return Optional.empty();
+    }
+
+    protected int indexOfServer(BookieServer b) throws Exception {
+        for (int i = 0; i < servers.size(); i++) {
+            if (servers.get(i).getServer().equals(b)) {
+                return i;
+            }
+        }
+        return -1;
+    }
+
+    protected int lastBookieIndex() {
+        return servers.size() - 1;
+    }
+
+    protected int bookieCount() {
+        return servers.size();
+    }
+
+    private OptionalInt indexByAddress(BookieId addr) throws UnknownHostException {
+        for (int i = 0; i < servers.size(); i++) {
+            if (addr.equals(servers.get(i).getServer().getBookieId())) {
+                return OptionalInt.of(i);
+            }
+        }
+        return OptionalInt.empty();
     }
 
     /**
      * Get bookie configuration for bookie.
      */
     public ServerConfiguration getBkConf(BookieId addr) throws Exception {
-        int bkIndex = 0;
-        for (BookieServer server : bs) {
-            if (server.getBookieId().equals(addr)) {
-                break;
-            }
-            ++bkIndex;
-        }
-        if (bkIndex < bs.size()) {
-            return bsConfs.get(bkIndex);
-        }
-        return null;
+        return byAddress(addr).get().getConfiguration();
     }
 
     /**
@@ -392,21 +425,16 @@ public abstract class BookKeeperClusterTestCase {
      * @throws InterruptedException
      */
     public ServerConfiguration killBookie(BookieId addr) throws Exception {
-        BookieServer toRemove = null;
-        int toRemoveIndex = 0;
-        for (BookieServer server : bs) {
-            if (server.getBookieId().equals(addr)) {
-                server.shutdown();
-                toRemove = server;
-                break;
+        Optional<ServerTester> tester = byAddress(addr);
+        if (tester.isPresent()) {
+            if (tester.get().autoRecovery != null
+                    && tester.get().autoRecovery.getAuditor() != null
+                    && tester.get().autoRecovery.getAuditor().isRunning()) {
+                LOG.warn("Killing bookie {} who is the current Auditor", addr);
             }
-            ++toRemoveIndex;
-        }
-        if (toRemove != null) {
-            stopAutoRecoveryService(toRemove);
-            bs.remove(toRemove);
-            bsLoggers.remove(addr);
-            return bsConfs.remove(toRemoveIndex);
+            servers.remove(tester.get());
+            tester.get().shutdown();
+            return tester.get().getConfiguration();
         }
         return null;
     }
@@ -418,12 +446,10 @@ public abstract class BookKeeperClusterTestCase {
      *          Socket Address
      * @throws InterruptedException
      */
-    public void setBookieToReadOnly(BookieId addr) throws InterruptedException, UnknownHostException {
-        for (BookieServer server : bs) {
-            if (server.getBookieId().equals(addr)) {
-                server.getBookie().getStateManager().doTransitionToReadOnlyMode();
-                break;
-            }
+    public void setBookieToReadOnly(BookieId addr) throws Exception {
+        Optional<ServerTester> tester = byAddress(addr);
+        if (tester.isPresent()) {
+            tester.get().getServer().getBookie().getStateManager().transitionToReadOnlyMode().get();
         }
     }
 
@@ -438,15 +464,9 @@ public abstract class BookKeeperClusterTestCase {
      * @throws IOException
      */
     public ServerConfiguration killBookie(int index) throws Exception {
-        if (index >= bs.size()) {
-            throw new IOException("Bookie does not exist");
-        }
-        BookieServer server = bs.get(index);
-        server.shutdown();
-        stopAutoRecoveryService(server);
-        bs.remove(server);
-        bsLoggers.remove(server.getBookieId());
-        return bsConfs.remove(index);
+        ServerTester tester = servers.remove(index);
+        tester.shutdown();
+        return tester.getConfiguration();
     }
 
     /**
@@ -457,13 +477,10 @@ public abstract class BookKeeperClusterTestCase {
      * @return configuration of killed bookie
      */
     public ServerConfiguration killBookieAndWaitForZK(int index) throws Exception {
-        if (index >= bs.size()) {
-            throw new IOException("Bookie does not exist");
-        }
-        BookieServer server = bs.get(index);
+        ServerTester tester = servers.get(index); // IKTODO: this method is awful
         ServerConfiguration ret = killBookie(index);
         while (zkc.exists(ZKMetadataDriverBase.resolveZkLedgersRootPath(baseConf) + "/" + AVAILABLE_NODE + "/"
-                + server.getBookieId().toString(), false) != null) {
+                       + tester.getServer().getBookieId().toString(), false) != null) {
             Thread.sleep(500);
         }
         return ret;
@@ -482,29 +499,29 @@ public abstract class BookKeeperClusterTestCase {
      */
     public CountDownLatch sleepBookie(BookieId addr, final int seconds)
             throws Exception {
-        for (final BookieServer bookie : bs) {
-            if (bookie.getBookieId().equals(addr)) {
-                final CountDownLatch l = new CountDownLatch(1);
-                Thread sleeper = new Thread() {
-                        @Override
-                        public void run() {
-                            try {
-                                bookie.suspendProcessing();
-                                LOG.info("bookie {} is asleep", bookie.getBookieId());
-                                l.countDown();
-                                Thread.sleep(seconds * 1000);
-                                bookie.resumeProcessing();
-                                LOG.info("bookie {} is awake", bookie.getBookieId());
-                            } catch (Exception e) {
-                                LOG.error("Error suspending bookie", e);
-                            }
+        Optional<ServerTester> tester = byAddress(addr);
+        if (tester.isPresent()) {
+            CountDownLatch latch = new CountDownLatch(1);
+            Thread sleeper = new Thread() {
+                    @Override
+                    public void run() {
+                        try {
+                            tester.get().getServer().suspendProcessing();
+                            LOG.info("bookie {} is asleep", tester.get().getAddress());
+                            latch.countDown();
+                            Thread.sleep(seconds * 1000);
+                            tester.get().getServer().resumeProcessing();
+                            LOG.info("bookie {} is awake", tester.get().getAddress());
+                        } catch (Exception e) {
+                            LOG.error("Error suspending bookie", e);
                         }
-                    };
-                sleeper.start();
-                return l;
-            }
+                    }
+                };
+            sleeper.start();
+            return latch;
+        } else {
+            throw new IOException("Bookie not found");
         }
-        throw new IOException("Bookie not found");
     }
 
     /**
@@ -526,10 +543,11 @@ public abstract class BookKeeperClusterTestCase {
 
     public void sleepBookie(BookieId addr, final CountDownLatch l, final CountDownLatch suspendLatch)
             throws InterruptedException, IOException {
-        for (final BookieServer bookie : bs) {
-            if (bookie.getBookieId().equals(addr)) {
-                LOG.info("Sleep bookie {}.", addr);
-                Thread sleeper = new Thread() {
+        Optional<ServerTester> tester = byAddress(addr);
+        if (tester.isPresent()) {
+            BookieServer bookie = tester.get().getServer();
+            LOG.info("Sleep bookie {}.", addr);
+            Thread sleeper = new Thread() {
                     @Override
                     public void run() {
                         try {
@@ -544,11 +562,10 @@ public abstract class BookKeeperClusterTestCase {
                         }
                     }
                 };
-                sleeper.start();
-                return;
-            }
+            sleeper.start();
+        } else {
+            throw new IOException("Bookie not found");
         }
-        throw new IOException("Bookie not found");
     }
 
     /**
@@ -562,7 +579,7 @@ public abstract class BookKeeperClusterTestCase {
      */
     public void restartBookies()
             throws Exception {
-        restartBookies(null);
+        restartBookies(c -> c);
     }
 
     /**
@@ -576,61 +593,30 @@ public abstract class BookKeeperClusterTestCase {
      * @throws BookieException
      */
     public void restartBookie(BookieId addr) throws Exception {
-        BookieServer toRemove = null;
-        int toRemoveIndex = 0;
-        for (BookieServer server : bs) {
-            if (server.getBookieId().equals(addr)) {
-                toRemove = server;
-                break;
-            }
-            ++toRemoveIndex;
-        }
-        if (toRemove != null) {
-            ServerConfiguration newConfig = bsConfs.get(toRemoveIndex);
-            killBookie(toRemoveIndex);
+        OptionalInt toRemove = indexByAddress(addr);
+        if (toRemove.isPresent()) {
+            ServerConfiguration newConfig = killBookie(toRemove.getAsInt());
             Thread.sleep(1000);
-            bs.add(startBookie(newConfig));
-            bsConfs.add(newConfig);
-            return;
+            startAndAddBookie(newConfig);
+        } else {
+            throw new IOException("Bookie not found");
         }
-        throw new IOException("Bookie not found");
     }
 
-    /**
-     * Restart bookie servers using new configuration settings. Also restart the
-     * respective auto recovery process, if isAutoRecoveryEnabled is true.
-     *
-     * @param newConf
-     *            New Configuration Settings
-     * @throws InterruptedException
-     * @throws IOException
-     * @throws KeeperException
-     * @throws BookieException
-     */
-    public void restartBookies(ServerConfiguration newConf)
+    public void restartBookies(Function<ServerConfiguration, ServerConfiguration> reconfFunction)
             throws Exception {
         // shut down bookie server
-        for (BookieServer server : bs) {
+        List<ServerConfiguration> confs = new ArrayList<>();
+        for (ServerTester server : servers) {
             server.shutdown();
-            stopAutoRecoveryService(server);
+            confs.add(server.getConfiguration());
         }
-        bs.clear();
-        bsLoggers.clear();
+        servers.clear();
         Thread.sleep(1000);
         // restart them to ensure we can't
-        for (ServerConfiguration conf : bsConfs) {
-            String bookieId = conf.getBookieId();
-            // ensure the bookie id or port is loaded correctly
-            int port = conf.getBookiePort();
-            if (null != newConf) {
-                conf.loadConf(newConf);
-            }
-            if (bookieId != null) {
-                conf.setBookieId(bookieId);
-            } else {
-                conf.setBookiePort(port);
-            }
-            bs.add(startBookie(conf));
+        for (ServerConfiguration conf : confs) {
+            // ensure the bookie port is loaded correctly
+            startAndAddBookie(reconfFunction.apply(conf));
         }
     }
 
@@ -643,25 +629,34 @@ public abstract class BookKeeperClusterTestCase {
      */
     public int startNewBookie()
             throws Exception {
+        return startNewBookieAndReturnAddress().getPort();
+    }
+
+    public BookieSocketAddress startNewBookieAndReturnAddress()
+            throws Exception {
         ServerConfiguration conf = newServerConfiguration();
-        bsConfs.add(conf);
         LOG.info("Starting new bookie on port: {}", conf.getBookiePort());
-        BookieServer server = startBookie(conf);
-        bs.add(server);
-        return server.getLocalAddress().getPort();
+        return startAndAddBookie(conf).getServer().getLocalAddress();
     }
 
     public BookieId startNewBookieAndReturnBookieId()
             throws Exception {
         ServerConfiguration conf = newServerConfiguration();
-        bsConfs.add(conf);
         LOG.info("Starting new bookie on port: {}", conf.getBookiePort());
-        BookieServer server = startBookie(conf);
-        bs.add(server);
-
-        return server.getBookieId();
+        return startAndAddBookie(conf).getServer().getBookieId();
     }
 
+    protected ServerTester startAndAddBookie(ServerConfiguration conf) throws Exception {
+        ServerTester server = startBookie(conf);
+        servers.add(server);
+        return server;
+    }
+
+    protected ServerTester startAndAddBookie(ServerConfiguration conf, Bookie b) throws Exception {
+        ServerTester server = startBookie(conf, b);
+        servers.add(server);
+        return server;
+    }
     /**
      * Helper method to startup a bookie server using a configuration object.
      * Also, starts the auto recovery process if isAutoRecoveryEnabled is true.
@@ -670,73 +665,54 @@ public abstract class BookKeeperClusterTestCase {
      *            Server Configuration Object
      *
      */
-    protected BookieServer startBookie(ServerConfiguration conf)
+    protected ServerTester startBookie(ServerConfiguration conf)
             throws Exception {
-        TestStatsProvider provider = new TestStatsProvider();
-        BookieServer server = new BookieServer(conf, provider.getStatsLogger(""), null);
-        BookieId address = Bookie.getBookieId(conf);
-        bsLoggers.put(address, provider);
+        ServerTester tester = new ServerTester(conf);
 
         if (bkc == null) {
             bkc = new BookKeeperTestClient(baseClientConf, new TestStatsProvider());
         }
 
+        BookieId address = tester.getServer().getBookieId();
         Future<?> waitForBookie = conf.isForceReadOnlyBookie()
             ? bkc.waitForReadOnlyBookie(address)
             : bkc.waitForWritableBookie(address);
 
-        server.start();
+        tester.getServer().start();
 
         waitForBookie.get(30, TimeUnit.SECONDS);
         LOG.info("New bookie '{}' has been created.", address);
 
-        try {
-            startAutoRecovery(server, conf);
-        } catch (CompatibilityException ce) {
-            LOG.error("Exception while starting AutoRecovery!", ce);
-        } catch (UnavailableException ue) {
-            LOG.error("Exception while starting AutoRecovery!", ue);
+        if (isAutoRecoveryEnabled()) {
+            tester.startAutoRecovery();
         }
-        return server;
+        return tester;
     }
 
     /**
      * Start a bookie with the given bookie instance. Also, starts the auto
      * recovery for this bookie, if isAutoRecoveryEnabled is true.
      */
-    protected BookieServer startBookie(ServerConfiguration conf, final Bookie b)
+    protected ServerTester startBookie(ServerConfiguration conf, final Bookie b)
             throws Exception {
-        TestStatsProvider provider = new TestStatsProvider();
-        BookieServer server = new BookieServer(conf, provider.getStatsLogger(""), null) {
-            @Override
-            protected Bookie newBookie(ServerConfiguration conf, ByteBufAllocator allocator,
-                                       Supplier<BookieServiceInfo> s) {
-                return b;
-            }
-        };
-
-        BookieId address = Bookie.getBookieId(conf);
+        ServerTester tester = new ServerTester(conf, b);
         if (bkc == null) {
             bkc = new BookKeeperTestClient(baseClientConf, new TestStatsProvider());
         }
+        BookieId address = tester.getServer().getBookieId();
         Future<?> waitForBookie = conf.isForceReadOnlyBookie()
             ? bkc.waitForReadOnlyBookie(address)
             : bkc.waitForWritableBookie(address);
 
-        server.start();
-        bsLoggers.put(server.getBookieId(), provider);
+        tester.getServer().start();
 
         waitForBookie.get(30, TimeUnit.SECONDS);
         LOG.info("New bookie '{}' has been created.", address);
 
-        try {
-            startAutoRecovery(server, conf);
-        } catch (CompatibilityException ce) {
-            LOG.error("Exception while starting AutoRecovery!", ce);
-        } catch (UnavailableException ue) {
-            LOG.error("Exception while starting AutoRecovery!", ue);
+        if (isAutoRecoveryEnabled()) {
+            tester.startAutoRecovery();
         }
-        return server;
+        return tester;
     }
 
     public void setMetastoreImplClass(AbstractConfiguration conf) {
@@ -767,36 +743,14 @@ public abstract class BookKeeperClusterTestCase {
         return isAutoRecoveryEnabled;
     }
 
-    private void startAutoRecovery(BookieServer bserver,
-                                   ServerConfiguration conf) throws Exception {
-        if (isAutoRecoveryEnabled()) {
-            AutoRecoveryMain autoRecoveryProcess = new AutoRecoveryMain(conf);
-            autoRecoveryProcess.start();
-            autoRecoveryProcesses.put(bserver, autoRecoveryProcess);
-            LOG.debug("Starting Auditor Recovery for the bookie:"
-                    + bserver.getBookieId());
-        }
-    }
-
-    private void stopAutoRecoveryService(BookieServer toRemove) throws Exception {
-        AutoRecoveryMain autoRecoveryMain = autoRecoveryProcesses
-                .remove(toRemove);
-        if (null != autoRecoveryMain && isAutoRecoveryEnabled()) {
-            autoRecoveryMain.shutdown();
-            LOG.debug("Shutdown auto recovery for bookieserver:"
-                    + toRemove.getBookieId());
-        }
-    }
-
     /**
      * Will starts the auto recovery process for the bookie servers. One auto
      * recovery process per each bookie server, if isAutoRecoveryEnabled is
      * enabled.
      */
     public void startReplicationService() throws Exception {
-        int index = -1;
-        for (BookieServer bserver : bs) {
-            startAutoRecovery(bserver, bsConfs.get(++index));
+        for (ServerTester t : servers) {
+            t.startAutoRecovery();
         }
     }
 
@@ -805,29 +759,30 @@ public abstract class BookKeeperClusterTestCase {
      * isAutoRecoveryEnabled is true.
      */
     public void stopReplicationService() throws Exception{
-        if (!isAutoRecoveryEnabled()){
-            return;
-        }
-        for (Entry<BookieServer, AutoRecoveryMain> autoRecoveryProcess : autoRecoveryProcesses
-                .entrySet()) {
-            autoRecoveryProcess.getValue().shutdown();
-            LOG.debug("Shutdown Auditor Recovery for the bookie:"
-                    + autoRecoveryProcess.getKey().getBookieId());
+        for (ServerTester t : servers) {
+            t.stopAutoRecovery();
         }
     }
 
     public Auditor getAuditor(int timeout, TimeUnit unit) throws Exception {
         final long timeoutAt = System.nanoTime() + TimeUnit.NANOSECONDS.convert(timeout, unit);
         while (System.nanoTime() < timeoutAt) {
-            for (AutoRecoveryMain p : autoRecoveryProcesses.values()) {
-                Auditor a = p.getAuditor();
-                ReplicationWorker replicationWorker = p.getReplicationWorker();
-                if (a != null && a.isRunning() && replicationWorker != null && replicationWorker.isRunning()) {
-                    int deathWatchInterval = baseConf.getDeathWatchInterval();
+            for (ServerTester t : servers) {
+                Auditor a = t.getAuditor();
+                ReplicationWorker replicationWorker = t.getReplicationWorker();
+
+                // found a candidate Auditor + ReplicationWorker
+                if (a != null && a.isRunning()
+                    && replicationWorker != null && replicationWorker.isRunning()) {
+                    int deathWatchInterval = t.getConfiguration().getDeathWatchInterval();
                     Thread.sleep(deathWatchInterval + 1000);
                 }
 
-                if (a != null && a.isRunning() && replicationWorker != null && replicationWorker.isRunning()) {
+                // double check, because in the meantime AutoRecoveryDeathWatcher may have killed the
+                // AutoRecovery daemon
+                if (a != null && a.isRunning()
+                        && replicationWorker != null && replicationWorker.isRunning()) {
+                    LOG.info("Found Auditor Bookie {}", t.server.getBookieId());
                     return a;
                 }
             }
@@ -836,10 +791,6 @@ public abstract class BookKeeperClusterTestCase {
         throw new Exception("No auditor found");
     }
 
-    public AutoRecoveryMain getAutoRecovery(BookieServer bServer) throws Exception {
-        return autoRecoveryProcesses.get(bServer);
-    }
-
     /**
      * Check whether the InetSocketAddress was created using a hostname or an IP
      * address. Represent as 'hostname/IPaddress' if the InetSocketAddress was
@@ -856,17 +807,161 @@ public abstract class BookKeeperClusterTestCase {
     }
 
     public void resetBookieOpLoggers() {
-        for (TestStatsProvider provider : bsLoggers.values()) {
-            provider.clear();
-        }
+        servers.forEach(t -> t.getStatsProvider().clear());
     }
 
-    public TestStatsProvider getStatsProvider(BookieId addr) {
-        return bsLoggers.get(addr);
+    public TestStatsProvider getStatsProvider(BookieId addr) throws UnknownHostException {
+        return byAddress(addr).get().getStatsProvider();
     }
 
     public TestStatsProvider getStatsProvider(int index) throws Exception {
-        return getStatsProvider(bs.get(index).getBookieId());
+        return servers.get(index).getStatsProvider();
     }
 
+    /**
+     * Class to encapsulate all the test objects.
+     */
+    public class ServerTester {
+        private final ServerConfiguration conf;
+        private final TestStatsProvider provider;
+        private final Bookie bookie;
+        private final BookieServer server;
+        private final BookieSocketAddress address;
+        private final MetadataBookieDriver metadataDriver;
+        private final RegistrationManager registrationManager;
+        private final LedgerManagerFactory lmFactory;
+        private final LedgerManager ledgerManager;
+        private final LedgerStorage storage;
+
+        private AutoRecoveryMain autoRecovery;
+
+        ServerTester(ServerConfiguration conf) throws Exception {
+            this.conf = conf;
+            provider = new TestStatsProvider();
+
+            StatsLogger rootStatsLogger = provider.getStatsLogger("");
+            StatsLogger bookieStats = rootStatsLogger.scope(BOOKIE_SCOPE);
+
+            metadataDriver = BookieResources.createMetadataDriver(conf, bookieStats);
+            registrationManager = metadataDriver.createRegistrationManager();
+            lmFactory = metadataDriver.getLedgerManagerFactory();
+            ledgerManager = lmFactory.newLedgerManager();
+
+            LegacyCookieValidation cookieValidation = new LegacyCookieValidation(
+                    conf, registrationManager);
+            cookieValidation.checkCookies(Main.storageDirectoriesFromConf(conf));
+
+            DiskChecker diskChecker = BookieResources.createDiskChecker(conf);
+            LedgerDirsManager ledgerDirsManager = BookieResources.createLedgerDirsManager(
+                    conf, diskChecker, bookieStats.scope(LD_LEDGER_SCOPE));
+            LedgerDirsManager indexDirsManager = BookieResources.createIndexDirsManager(
+                    conf, diskChecker, bookieStats.scope(LD_INDEX_SCOPE), ledgerDirsManager);
+
+            UncleanShutdownDetection uncleanShutdownDetection = new UncleanShutdownDetectionImpl(ledgerDirsManager);
+
+            storage = BookieResources.createLedgerStorage(
+                    conf, ledgerManager, ledgerDirsManager, indexDirsManager,
+                    bookieStats, allocator);
+
+            if (conf.isForceReadOnlyBookie()) {
+                bookie = new ReadOnlyBookie(conf, registrationManager, storage,
+                                            diskChecker, ledgerDirsManager, indexDirsManager,
+                                            bookieStats, allocator, BookieServiceInfo.NO_INFO);
+            } else {
+                bookie = new BookieImpl(conf, registrationManager, storage,
+                                        diskChecker, ledgerDirsManager, indexDirsManager,
+                                        bookieStats, allocator, BookieServiceInfo.NO_INFO);
+            }
+            server = new BookieServer(conf, bookie, rootStatsLogger, allocator,
+                    uncleanShutdownDetection);
+            address = BookieImpl.getBookieAddress(conf);
+
+            autoRecovery = null;
+        }
+
+        ServerTester(ServerConfiguration conf, Bookie b) throws Exception {
+            this.conf = conf;
+            provider = new TestStatsProvider();
+
+            metadataDriver = null;
+            registrationManager = null;
+            ledgerManager = null;
+            lmFactory = null;
+            storage = null;
+
+            bookie = b;
+            server = new BookieServer(conf, b, provider.getStatsLogger(""),
+                    allocator, new MockUncleanShutdownDetection());
+            address = BookieImpl.getBookieAddress(conf);
+
+            autoRecovery = null;
+        }
+
+        void startAutoRecovery() throws Exception {
+            LOG.debug("Starting Auditor Recovery for the bookie: {}", address);
+            autoRecovery = new AutoRecoveryMain(conf);
+            autoRecovery.start();
+        }
+
+        void stopAutoRecovery() {
+            if (autoRecovery != null) {
+                LOG.debug("Shutdown Auditor Recovery for the bookie: {}", address);
+                autoRecovery.shutdown();
+            }
+        }
+
+        Auditor getAuditor() {
+            if (autoRecovery != null) {
+                return autoRecovery.getAuditor();
+            } else {
+                return null;
+            }
+        }
+
+        ReplicationWorker getReplicationWorker() {
+            if (autoRecovery != null) {
+                return autoRecovery.getReplicationWorker();
+            } else {
+                return null;
+            }
+        }
+
+        ServerConfiguration getConfiguration() {
+            return conf;
+        }
+
+        public BookieServer getServer() {
+            return server;
+        }
+
+        TestStatsProvider getStatsProvider() {
+            return provider;
+        }
+
+        BookieSocketAddress getAddress() {
+            return address;
+        }
+
+        void shutdown() throws Exception {
+            server.shutdown();
+
+            if (ledgerManager != null) {
+                ledgerManager.close();
+            }
+            if (lmFactory != null) {
+                lmFactory.close();
+            }
+            if (registrationManager != null) {
+                registrationManager.close();
+            }
+            if (metadataDriver != null) {
+                metadataDriver.close();
+            }
+
+            if (autoRecovery != null) {
+                LOG.debug("Shutdown auto recovery for bookieserver: {}", address);
+                autoRecovery.shutdown();
+            }
+        }
+    }
 }
